@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -66,7 +66,7 @@ import org.hsqldb.types.TimestampData;
  * protocol.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.5
+ * @version 2.5.0
  * @since 1.7.2
  */
 public class ClientConnection implements SessionInterface, Cloneable {
@@ -683,7 +683,7 @@ public class ClientConnection implements SessionInterface, Cloneable {
      */
     public static String toNetCompVersionString(int i) {
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
 
         i *= -1;
 
@@ -711,4 +711,8 @@ public class ClientConnection implements SessionInterface, Cloneable {
         dataOutput.writeInt(NETWORK_COMPATIBILITY_VERSION_INT);
         dataOutput.flush();
     }
+
+    public Object clone() throws CloneNotSupportedException {
+        return super.clone();
+    }
 }
