@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2018, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -61,7 +61,7 @@ import org.hsqldb.types.Collation;
  * It holds the data structures that form an HSQLDB database instance.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.4.1
+ * @version 2.5.0
  * @since 1.9.0
  */
 public class Database {
@@ -96,7 +96,7 @@ public class Database {
      *  databaseProperties or URL properties for new databases.
      */
     public int                    sqlAvgScale            = 0;
-    public boolean                sqlRestrictExec          = false;
+    public boolean                sqlRestrictExec        = false;
     public boolean                sqlCharLiteral         = true;
     public boolean                sqlConcatNulls         = true;
     public boolean                sqlConvertTruncate     = true;
@@ -136,7 +136,8 @@ public class Database {
     public SessionManager     sessionManager;
     public TransactionManager txManager;
     public int defaultIsolationLevel = SessionInterface.TX_READ_COMMITTED;
-    public boolean            txConflictRollback = true;
+    public boolean            txConflictRollback  = true;
+    public boolean            txInterruptRollback = false;
 
     // schema objects
     public SchemaManager schemaManager;
@@ -582,22 +583,6 @@ public class Database {
         sqlSyntaxPgs = mode;
     }
 
-    /**
-     *  Called by the garbage collector on this Databases object when garbage
-     *  collection determines that there are no more references to it.
-     */
-    protected void finalize() {
-
-        if (getState() != DATABASE_ONLINE) {
-            return;
-        }
-
-        try {
-            close(CLOSEMODE_IMMEDIATELY);
-        } catch (HsqlException e) {    // it's too late now
-        }
-    }
-
     void closeIfLast() {
 
         if (sessionManager.isEmpty() && dbState == DATABASE_ONLINE) {
@@ -727,7 +712,7 @@ public class Database {
     public String[] getSettingsSQL() {
 
         HsqlArrayList list = new HsqlArrayList();
-        StringBuffer  sb   = new StringBuffer();
+        StringBuilder sb   = new StringBuilder();
 
         if (!getCatalogName().name.equals(
                 SqlInvariants.DEFAULT_CATALOG_NAME)) {
@@ -738,7 +723,7 @@ public class Database {
             sb.setLength(0);
         }
 
-        if (!collation.isDefaultCollation()) {
+        if (!collation.isDefaultCollation() || !collation.isPadSpace()) {
             list.add(collation.getDatabaseCollationSQL());
         }
 
@@ -933,13 +918,6 @@ public class Database {
             }
         }
 
-        public void start() {
-
-            sessionList = new OrderedHashSet();
-            timerTask = DatabaseManager.getTimer().schedulePeriodicallyAfter(0,
-                    1000, this, true);
-        }
-
         public void stop() {
 
             synchronized (this) {
@@ -965,5 +943,12 @@ public class Database {
                 sessionList.add(session);
             }
         }
+
+        private void start() {
+
+            sessionList = new OrderedHashSet();
+            timerTask = DatabaseManager.getTimer().schedulePeriodicallyAfter(0,
+                    1000, this, true);
+        }
     }
 }
