@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2018, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -61,7 +61,7 @@ import org.hsqldb.types.Types;
  *
  * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.4.1
+ * @version 2.5.0
  * @since 1.9.0
  */
 public class Expression implements Cloneable {
@@ -81,70 +81,6 @@ public class Expression implements Cloneable {
     static final Expression EXPR_TRUE  = new ExpressionLogical(true);
     static final Expression EXPR_FALSE = new ExpressionLogical(false);
 
-    //
-    static final OrderedIntHashSet aggregateFunctionSet =
-        new OrderedIntHashSet();
-
-    static {
-        aggregateFunctionSet.add(OpTypes.COUNT);
-        aggregateFunctionSet.add(OpTypes.AVG);
-        aggregateFunctionSet.add(OpTypes.MAX);
-        aggregateFunctionSet.add(OpTypes.MIN);
-        aggregateFunctionSet.add(OpTypes.SUM);
-        aggregateFunctionSet.add(OpTypes.EVERY);
-        aggregateFunctionSet.add(OpTypes.SOME);
-        aggregateFunctionSet.add(OpTypes.STDDEV_POP);
-        aggregateFunctionSet.add(OpTypes.STDDEV_SAMP);
-        aggregateFunctionSet.add(OpTypes.VAR_POP);
-        aggregateFunctionSet.add(OpTypes.VAR_SAMP);
-        aggregateFunctionSet.add(OpTypes.ARRAY_AGG);
-        aggregateFunctionSet.add(OpTypes.USER_AGGREGATE);
-        aggregateFunctionSet.add(OpTypes.GROUP_CONCAT);
-        aggregateFunctionSet.add(OpTypes.MEDIAN);
-    }
-
-    static final OrderedIntHashSet columnExpressionSet =
-        new OrderedIntHashSet();
-
-    static {
-        columnExpressionSet.add(OpTypes.COLUMN);
-    }
-
-    static final OrderedIntHashSet subqueryExpressionSet =
-        new OrderedIntHashSet();
-
-    static {
-        subqueryExpressionSet.add(OpTypes.ROW_SUBQUERY);
-        subqueryExpressionSet.add(OpTypes.TABLE_SUBQUERY);
-    }
-
-    static final OrderedIntHashSet subqueryAggregateExpressionSet =
-        new OrderedIntHashSet();
-
-    static {
-        subqueryAggregateExpressionSet.addAll(aggregateFunctionSet);
-        subqueryAggregateExpressionSet.addAll(subqueryExpressionSet);
-    }
-
-    static final OrderedIntHashSet functionExpressionSet =
-        new OrderedIntHashSet();
-
-    static {
-        functionExpressionSet.add(OpTypes.SQL_FUNCTION);
-        functionExpressionSet.add(OpTypes.FUNCTION);
-    }
-
-    static final OrderedIntHashSet sequenceExpressionSet =
-        new OrderedIntHashSet();
-
-    static {
-        sequenceExpressionSet.add(OpTypes.ROWNUM);
-        sequenceExpressionSet.add(OpTypes.SEQUENCE);
-    }
-
-    static final OrderedIntHashSet emptyExpressionSet =
-        new OrderedIntHashSet();
-
     // type
     protected int opType;
 
@@ -159,7 +95,9 @@ public class Expression implements Cloneable {
     boolean         isDistinctAggregate;
 
     // VALUE
-    protected Object       valueData;
+    protected Object valueData;
+
+    //
     protected Expression[] nodes;
     Type[]                 nodeDataTypes;
     TableDerived           table;
@@ -182,9 +120,6 @@ public class Expression implements Cloneable {
     // index of a session-dependent field
     int parameterIndex = -1;
 
-    //
-    int rangePosition = -1;
-
     //
     boolean isColumnCondition;
     boolean isColumnEqual;
@@ -269,7 +204,7 @@ public class Expression implements Cloneable {
                 return ddl;
         }
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
 
         ddl = sb.append('(').append(ddl).append(')').toString();
 
@@ -295,7 +230,7 @@ public class Expression implements Cloneable {
      */
     public String getSQL() {
 
-        StringBuffer sb = new StringBuffer(64);
+        StringBuilder sb = new StringBuilder(64);
 
         switch (opType) {
 
@@ -368,7 +303,7 @@ public class Expression implements Cloneable {
 
     protected String describe(Session session, int blanks) {
 
-        StringBuffer sb = new StringBuffer(64);
+        StringBuilder sb = new StringBuilder(64);
 
         sb.append('\n');
 
@@ -440,7 +375,7 @@ public class Expression implements Cloneable {
     /**
      * SIMPLE_COLUMN expressions can be of different Expression subclass types
      */
-    public boolean equals(Expression other) {
+    boolean equals(Expression other) {
 
         if (other == this) {
             return true;
@@ -474,7 +409,7 @@ public class Expression implements Cloneable {
         }
     }
 
-    public boolean equals(Object other) {
+    public final boolean equals(Object other) {
 
         if (other == this) {
             return true;
@@ -605,7 +540,7 @@ public class Expression implements Cloneable {
             }
         }
 
-        if (aggregateFunctionSet.contains(opType)) {
+        if (OpTypes.aggregateFunctionSet.contains(opType)) {
             return false;
         }
 
@@ -687,7 +622,7 @@ public class Expression implements Cloneable {
         case OpCodes.TABLE_SUBQUERY :
         case OpCodes.ROW_SUBQUERY :
 */
-        if (aggregateFunctionSet.contains(opType)) {
+        if (OpTypes.aggregateFunctionSet.contains(opType)) {
             return false;
         }
 
@@ -706,7 +641,7 @@ public class Expression implements Cloneable {
         return result;
     }
 
-    Expression replaceColumnReferences(RangeVariable range,
+    Expression replaceColumnReferences(Session session, RangeVariable range,
                                        Expression[] list) {
 
         for (int i = 0; i < nodes.length; i++) {
@@ -714,11 +649,12 @@ public class Expression implements Cloneable {
                 continue;
             }
 
-            nodes[i] = nodes[i].replaceColumnReferences(range, list);
+            nodes[i] = nodes[i].replaceColumnReferences(session, range, list);
         }
 
         if (table != null && table.queryExpression != null) {
-            table.queryExpression.replaceColumnReferences(range, list);
+            table.queryExpression.replaceColumnReferences(session, range,
+                    list);
         }
 
         return this;
@@ -803,7 +739,7 @@ public class Expression implements Cloneable {
         hasAggregate = true;
     }
 
-    boolean isSelfAggregate() {
+    public boolean isSelfAggregate() {
         return false;
     }
 
@@ -870,7 +806,7 @@ public class Expression implements Cloneable {
     }
 
     /**
-     * Returns the range variable for a COLUMN expression
+     * Returns the range variable for a COLUMN or PERIOD expression
      */
     RangeVariable getRangeVariable() {
         return null;
@@ -1213,8 +1149,9 @@ public class Expression implements Cloneable {
                                     session, nodes[i].valueData);
                         } else {
                             nodes[i].valueData =
-                                nodeDataType.convertToType(
-                                    session, nodes[i].valueData, nodes[i].dataType);
+                                nodeDataType.convertToType(session,
+                                                           nodes[i].valueData,
+                                                           nodes[i].dataType);
                         }
                     }
                 }
@@ -1223,7 +1160,6 @@ public class Expression implements Cloneable {
                     nodes[i].dataType = nodeDataType;
                 }
 
-
                 dataType = new ArrayType(nodeDataType, nodes.length);
 
                 return;
@@ -1725,15 +1661,15 @@ public class Expression implements Cloneable {
 
         ExpressionColumn.checkColumnsResolved(unresolved);
         resolveTypes(session, null);
-        collectAllExpressions(set, Expression.subqueryAggregateExpressionSet,
-                              Expression.emptyExpressionSet);
+        collectAllExpressions(set, OpTypes.subqueryAggregateExpressionSet,
+                              OpTypes.emptyExpressionSet);
 
         if (!set.isEmpty()) {
             throw Error.error(ErrorCode.X_42512);
         }
 
-        collectAllExpressions(set, Expression.functionExpressionSet,
-                              Expression.emptyExpressionSet);
+        collectAllExpressions(set, OpTypes.functionExpressionSet,
+                              OpTypes.emptyExpressionSet);
 
         for (int i = 0; i < set.size(); i++) {
             Expression current = (Expression) set.get(i);
@@ -1901,8 +1837,8 @@ public class Expression implements Cloneable {
 
         OrderedHashSet list = null;
 
-        list = collectAllExpressions(list, Expression.functionExpressionSet,
-                                     Expression.emptyExpressionSet);
+        list = collectAllExpressions(list, OpTypes.functionExpressionSet,
+                                     OpTypes.emptyExpressionSet);
 
         if (list == null) {
             return false;
@@ -2036,8 +1972,8 @@ public class Expression implements Cloneable {
         OrderedHashSet set = null;
 
         set = collectAllExpressions(set,
-                                    Expression.subqueryAggregateExpressionSet,
-                                    Expression.emptyExpressionSet);
+                                    OpTypes.subqueryAggregateExpressionSet,
+                                    OpTypes.emptyExpressionSet);
 
         if (set != null && !set.isEmpty()) {
             throw Error.error(ErrorCode.X_0A000,
@@ -2095,7 +2031,7 @@ public class Expression implements Cloneable {
 
         try {
             e       = (Expression) super.clone();
-            e.nodes = (Expression[]) nodes.clone();
+            e.nodes = nodes.clone();
 
             for (int i = 0; i < nodes.length; i++) {
                 if (nodes[i] != null) {
@@ -2128,8 +2064,7 @@ public class Expression implements Cloneable {
         throw Error.runtimeError(ErrorCode.U_S0500, "Expression");
     }
 
-    public Object getAggregatedValue(Session session,
-                                     SetFunction currValue) {
+    public Object getAggregatedValue(Session session, SetFunction currValue) {
         throw Error.runtimeError(ErrorCode.U_S0500, "Expression");
     }
 
@@ -2141,7 +2076,7 @@ public class Expression implements Cloneable {
         return false;
     }
 
-    public void setCondition(Expression e) {
+    public void setCondition(ExpressionLogical e) {
         throw Error.runtimeError(ErrorCode.U_S0500, "Expression");
     }
 
