@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2018, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,10 +39,13 @@ import org.hsqldb.lib.HsqlList;
 import org.hsqldb.lib.OrderedHashSet;
 import org.hsqldb.map.ValuePool;
 import org.hsqldb.persist.HsqlDatabaseProperties;
+import org.hsqldb.persist.RowInsertInterface;
 import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultProperties;
 import org.hsqldb.rights.User;
 import org.hsqldb.types.Charset;
+import org.hsqldb.types.DateTimeType;
+import org.hsqldb.types.TimestampData;
 import org.hsqldb.types.Type;
 import org.hsqldb.types.Types;
 
@@ -50,7 +53,7 @@ import org.hsqldb.types.Types;
  * Parser for session and management statements
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.4.1
+ * @version 2.5.0
  * @since 1.9.0
  */
 public class ParserCommand extends ParserDDL {
@@ -277,7 +280,7 @@ public class ParserCommand extends ParserDDL {
 
             // HSQL COMMAND
             case Tokens.SCRIPT :
-                cs = compileScript();
+                cs = compileScript(false);
                 break;
 
             case Tokens.SHUTDOWN :
@@ -371,19 +374,100 @@ public class ParserCommand extends ParserDDL {
         return cs;
     }
 
-    private Statement compileScript() {
+    private Statement compileScript(boolean extended) {
 
-        String name = null;
+        String        name      = null;
+        int           scope     = 0;
+        int           type      = 0;
+        HsqlName      tableName = null;
+        TimestampData timestamp = null;
 
-        read();
+        readThis(Tokens.SCRIPT);
+
+        if (extended) {
+            readThis(Tokens.FOR);
+
+            if (readIfThis(Tokens.DATABASE)) {
+                switch (token.tokenType) {
+
+                    case Tokens.STRUCTURE :
+                        read();
+
+                        type = Tokens.STRUCTURE;
+                        break;
+
+                    case Tokens.VERSIONING :
+                        read();
+                        readThis(Tokens.DATA);
+
+                        if (readIfThis(Tokens.FROM)) {
+                            readThis(Tokens.TIMESTAMP);
+
+                            String s = readQuotedString();
+
+                            timestamp =
+                                (TimestampData) Type.SQL_TIMESTAMP
+                                    .convertToType(session, s,
+                                                   Type.SQL_VARCHAR_DEFAULT);
+                        } else {
+                            timestamp = DateTimeType.epochTimestamp;
+                        }
+
+                        type = Tokens.VERSIONING;
+                        break;
+
+                    case Tokens.DATA :
+                        read();
+
+                        type = Tokens.DATA;
+                        break;
+
+                    default :
+                        type = Tokens.ALL;
+                }
+
+                readThis(Tokens.TO);
+
+                scope = Tokens.DATABASE;
+            } else if (readIfThis(Tokens.TABLE)) {
+                Table table = readTableName();
+
+                if (table.isView() || table.isTemp()) {
+                    throw Error.error(ErrorCode.X_42501);
+                }
+
+                tableName = table.getName();
+
+                readThis(Tokens.DATA);
+                readThis(Tokens.TO);
+
+                scope = Tokens.TABLE;
+                type  = Tokens.DATA;
+            } else {
+                throw unexpectedToken();
+            }
+        }
 
         if (token.tokenType == Tokens.X_VALUE) {
+            if (scope == 0) {
+                scope = Tokens.DATABASE;
+                type  = Tokens.ALL;
+            }
+
             name = readQuotedString();
+        } else if (scope == 0) {
+            scope = Tokens.DATABASE;
+            type  = Tokens.STRUCTURE;
+        } else {
+            throw unexpectedTokenRequire(Tokens.T_PATH);
         }
 
         HsqlName[] names =
             database.schemaManager.getCatalogAndBaseTableNames();
-        Object[] args = new Object[]{ name };
+        Object[] args = new Object[] {
+            name, Integer.valueOf(scope), Integer.valueOf(type), tableName,
+            timestamp
+        };
 
         return new StatementCommand(StatementTypes.DATABASE_SCRIPT, args,
                                     null, names);
@@ -615,7 +699,7 @@ public class ParserCommand extends ParserDDL {
                             break;
                         }
 
-                    // fall through
+                        throw Error.error(ErrorCode.X_0P000);
                     default :
                         throw Error.error(ErrorCode.X_0P000);
                 }
@@ -1407,16 +1491,23 @@ public class ParserCommand extends ParserDDL {
                 if (readIfThis(Tokens.ROLLBACK)) {
                     readThis(Tokens.ON);
 
-                    if (!readIfThis(Tokens.DEADLOCK)) {
+                    if (readIfThis(Tokens.INTERRUPT)) {
+                        Boolean mode = processTrueOrFalseObject();
+                        StatementCommand cs = new StatementCommand(
+                            StatementTypes.SET_DATABASE_TRANSACTION_INTERRUPT,
+                            new Object[]{ mode }, null, null);
+
+                        return cs;
+                    } else {
                         readThis(Tokens.CONFLICT);
+
+                        Boolean mode = processTrueOrFalseObject();
+                        StatementCommand cs = new StatementCommand(
+                            StatementTypes.SET_DATABASE_TRANSACTION_CONFLICT,
+                            new Object[]{ mode }, null, null);
+
+                        return cs;
                     }
-
-                    Boolean mode = processTrueOrFalseObject();
-                    StatementCommand cs = new StatementCommand(
-                        StatementTypes.SET_DATABASE_TRANSACTION_CONFLICT,
-                        new Object[]{ mode }, null, null);
-
-                    return cs;
                 }
 
                 readThis(Tokens.CONTROL);
@@ -1887,7 +1978,6 @@ public class ParserCommand extends ParserDDL {
         OrderedHashSet readSet  = new OrderedHashSet();
         OrderedHashSet writeSet = new OrderedHashSet();
 
-        outerloop:
         while (true) {
             Table table = readTableName(true);
 
@@ -1913,7 +2003,7 @@ public class ParserCommand extends ParserDDL {
                 continue;
             }
 
-            break outerloop;
+            break;
         }
 
         HsqlName[] writeTableNames = new HsqlName[writeSet.size()];
@@ -2353,11 +2443,93 @@ public class ParserCommand extends ParserDDL {
                 return new StatementCommand(StatementTypes.CHECK_INDEX, args,
                                             null, names);
             }
+            case Tokens.IMPORT : {
+                read();
+
+                return compileImportScript();
+            }
+            case Tokens.EXPORT : {
+                return compileExport();
+            }
             default :
                 throw unexpectedToken();
         }
     }
 
+    private Statement compileExport() {
+
+        read();
+
+        return compileScript(true);
+    }
+
+    private Statement compileImportScript() {
+
+        String  fileName;
+        int     mode         = RowInsertInterface.modes.continueOnError;
+        Boolean isVersioning = Boolean.FALSE;
+
+        readThis(Tokens.SCRIPT);
+
+        if (token.tokenType == Tokens.VERSIONING) {
+            readThis(Tokens.VERSIONING);
+
+            isVersioning = Boolean.TRUE;
+        }
+
+        readThis(Tokens.DATA);
+        readThis(Tokens.FROM);
+
+        fileName = readQuotedString();
+
+        if (!isVersioning) {
+            mode = readLoadMode();
+        }
+
+        HsqlName[] names =
+            database.schemaManager.getCatalogAndBaseTableNames();
+        Object[] args = new Object[] {
+            fileName, Integer.valueOf(mode), isVersioning
+        };
+
+        return new StatementCommand(StatementTypes.LOAD_SCRIPT, args, null,
+                                    names);
+    }
+
+    private int readLoadMode() {
+
+        int mode = -1;
+
+        switch (token.tokenType) {
+
+            case Tokens.CONTINUE :
+                mode = RowInsertInterface.modes.continueOnError;
+
+                read();
+                break;
+
+            case Tokens.STOP :
+                mode = RowInsertInterface.modes.stopOnError;
+
+                read();
+                break;
+
+            case Tokens.CHECK :
+                mode = RowInsertInterface.modes.checkUntillError;
+
+                read();
+                break;
+
+            default :
+                throw unexpectedToken();
+        }
+
+        readThis(Tokens.ON);
+        readThis(Tokens.ERROR);
+
+        return mode;
+    }
+
     private Statement compileCheckpoint() {
 
         boolean defrag = false;
@@ -2405,9 +2577,8 @@ public class ParserCommand extends ParserDDL {
 
         read();
 
-        String sql = Tokens.T_DISCONNECT;
-        Statement cs = new StatementSession(StatementTypes.DISCONNECT,
-                                            (Object[]) null);
+        String    sql = Tokens.T_DISCONNECT;
+        Statement cs  = new StatementSession(StatementTypes.DISCONNECT, null);
 
         return cs;
     }
