@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2017, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -296,25 +296,25 @@ public class RowAction extends RowActionBase {
 
 //        memoryRow       = null;
         session         = null;
+        type            = ACTION_NONE;
         actionTimestamp = 0;
         commitTimestamp = 0;
         rolledback      = false;
         deleteComplete  = false;
-        changeColumnMap = null;
         prepared        = false;
-        type            = ACTION_NONE;
+        changeColumnMap = null;
         next            = null;
     }
 
     private void setAsDeleteFinal(long timestamp) {
 
+        type            = ACTION_DELETE_FINAL;
         actionTimestamp = 0;
         commitTimestamp = timestamp;
         rolledback      = false;
         deleteComplete  = false;
         prepared        = false;
         changeColumnMap = null;
-        type            = ACTION_DELETE_FINAL;
         next            = null;
     }
 
@@ -420,7 +420,7 @@ public class RowAction extends RowActionBase {
     /**
      * returns false if another committed session has altered the same row
      */
-    synchronized boolean canCommit(Session session, OrderedHashSet set) {
+    synchronized boolean canCommit(Session session) {
 
         RowActionBase action;
         long          timestamp       = session.transactionTimestamp;
@@ -465,7 +465,7 @@ public class RowAction extends RowActionBase {
                 }
 
                 if (action.commitTimestamp == 0) {
-                    set.add(action);
+                    session.actionSet.add(action);
                 } else if (action.commitTimestamp > commitTimestamp) {
                     commitTimestamp = action.commitTimestamp;
                 }
@@ -481,23 +481,6 @@ public class RowAction extends RowActionBase {
         return commitTimestamp < timestamp;
     }
 
-    synchronized void complete(Session session) {
-
-        RowActionBase action;
-
-        action = this;
-
-        do {
-            if (action.session == session) {
-                if (action.actionTimestamp == 0) {
-                    action.actionTimestamp = session.actionTimestamp;
-                }
-            }
-
-            action = action.next;
-        } while (action != null);
-    }
-
     /**
      * returns false if cannot complete
      * when READ COMMITTED, false result always means repeat action and adds
@@ -843,8 +826,6 @@ public class RowAction extends RowActionBase {
             }
 
             action = action.next;
-
-            continue;
         } while (action != null);
 
         if (actionType == ACTION_NONE || actionType == ACTION_INSERT) {
