@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,7 +62,7 @@ import org.hsqldb.types.Types;
  * Scans for SQL tokens.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.5
+ * @version 2.5.0
  * @since 1.9.0
  */
 public class Scanner {
@@ -372,13 +372,9 @@ public class Scanner {
 
         if (c >= '0' && c <= '9') {
             c -= '0';
-        } else if (c > 'z') {
-            c = 16;
-        } else if (c >= 'a') {
+        } else  if (c >= 'a' && c <= 'f') {
             c -= ('a' - 10);
-        } else if (c > 'Z') {
-            c = 16;
-        } else if (c >= 'A') {
+        } else  if (c >= 'A' && c <= 'F') {
             c -= ('A' - 10);
         } else {
             c = -1;
@@ -1132,20 +1128,21 @@ public class Scanner {
 
                 // fredt -  -Integer.MIN_VALUE or -Long.MIN_VALUE are promoted
                 // to a wider type.
-                if (token.tokenString.length() < 11) {
+                if (token.tokenString.length() < 20) {
                     try {
-                        token.tokenValue = ValuePool.getInt(
-                            Integer.parseInt(token.tokenString));
+                        long longVal = Long.parseLong(token.tokenString);
 
-                        return;
-                    } catch (Exception e1) {}
-                }
+                        if (token.tokenString.length() < 11) {
+                            if (longVal <= Integer.MAX_VALUE) {
+                                token.tokenValue =
+                                    ValuePool.getInt((int) longVal);
 
-                if (this.token.tokenString.length() < 20) {
-                    try {
-                        token.dataType = Type.SQL_BIGINT;
-                        token.tokenValue = ValuePool.getLong(
-                            Long.parseLong(token.tokenString));
+                                return;
+                            }
+                        }
+
+                        token.dataType   = Type.SQL_BIGINT;
+                        token.tokenValue = ValuePool.getLong(longVal);
 
                         return;
                     } catch (Exception e2) {}
@@ -1231,20 +1228,7 @@ public class Scanner {
 
             return true;
         } else if (character == '/' && charAt(currentPosition + 1) == '*') {
-            int pos = sqlString.indexOf("*/", currentPosition + 2);
-
-            if (pos == -1) {
-                token.tokenString = sqlString.substring(currentPosition,
-                        currentPosition + 2);
-                token.tokenType   = Tokens.X_MALFORMED_COMMENT;
-                token.isMalformed = true;
-
-                return false;
-            }
-
-            currentPosition = pos + 2;
-
-            return true;
+            return skipBracketedComment();
         }
 
         return false;
@@ -1297,6 +1281,35 @@ public class Scanner {
         }
     }
 
+    private static int countEndOfLines(String s) {
+
+        int eolPos    = -2;
+        int eolCode   = 0;
+        int lineCount = 0;
+
+        for (int i = 0; i < s.length(); i++) {
+            int c = s.charAt(i);
+
+            if (c == '\r' || c == '\n') {
+                if (i == eolPos + 1) {
+                    if (c == '\n' && eolCode != c) {
+
+                        //
+                    } else {
+                        lineCount++;
+                    }
+                } else {
+                    lineCount++;
+                }
+
+                eolPos  = i;
+                eolCode = c;
+            }
+        }
+
+        return lineCount;
+    }
+
     void scanCharacterString() {
 
         charWriter.reset(charBuffer);
@@ -1616,22 +1629,7 @@ public class Scanner {
 
                     return;
                 } else if (charAt(currentPosition + 1) == '*') {
-                    int pos = sqlString.indexOf("*/", currentPosition + 2);
-
-                    if (pos == -1) {
-                        token.tokenString =
-                            sqlString.substring(currentPosition,
-                                                currentPosition + 2);
-                        token.tokenType   = Tokens.X_UNKNOWN_TOKEN;
-                        token.isDelimiter = true;
-
-                        return;
-                    }
-
-                    token.tokenString = sqlString.substring(currentPosition
-                            + 2, pos);
-                    token.tokenType   = Tokens.X_REMARK;
-                    token.isDelimiter = true;
+                    scanBracketedComment();
 
                     return;
                 }
@@ -1862,6 +1860,48 @@ public class Scanner {
         setIdentifierProperties();
     }
 
+    private boolean skipBracketedComment() {
+
+        int pos = sqlString.indexOf("*/", currentPosition + 2);
+
+        if (pos == -1) {
+            token.tokenString = sqlString.substring(currentPosition,
+                    currentPosition + 2);
+            token.tokenType   = Tokens.X_MALFORMED_COMMENT;
+            token.isMalformed = true;
+
+            return false;
+        }
+
+        String comment = sqlString.substring(currentPosition + 2, pos);
+
+        currentPosition = pos + 2;
+        lineNumber      += countEndOfLines(comment);
+
+        return true;
+    }
+
+    private boolean scanBracketedComment() {
+
+        int pos = sqlString.indexOf("*/", currentPosition + 2);
+
+        if (pos == -1) {
+            token.tokenString = sqlString.substring(currentPosition,
+                    currentPosition + 2);
+            token.tokenType   = Tokens.X_UNKNOWN_TOKEN;
+            token.isDelimiter = true;
+
+            return false;
+        }
+
+        token.tokenString = sqlString.substring(currentPosition + 2, pos);
+        token.tokenType   = Tokens.X_REMARK;
+        token.isDelimiter = true;
+        lineNumber        += countEndOfLines(token.tokenString);
+
+        return true;
+    }
+
     private void setIdentifierProperties() {
 
         if (token.tokenType == Tokens.X_IDENTIFIER) {
@@ -2049,15 +2089,15 @@ public class Scanner {
 
         if (intervalPosition == 10) {
             seconds = HsqlDateTime.getDateSeconds(s.substring(0,
-                intervalPosition));
-
+                    intervalPosition));
             dateTimeType = Type.SQL_TIMESTAMP_NO_FRACTION;
 
             return new TimestampData(seconds, fraction, (int) zoneSeconds);
         } else {
             seconds = HsqlDateTime.getTimestampSeconds(s.substring(0,
-                intervalPosition));
+                    intervalPosition));
         }
+
         int position;
 
         fraction = scanIntervalFraction(DTIType.maxFractionPrecision);
@@ -2092,6 +2132,10 @@ public class Scanner {
             seconds -= zoneSeconds;
         }
 
+        if (seconds > DateTimeType.epochLimitSeconds) {
+            throw Error.error(ErrorCode.X_22008);
+        }
+
         return new TimestampData(seconds, fraction, (int) zoneSeconds);
     }
 
@@ -2137,7 +2181,7 @@ public class Scanner {
 
             if (endOfPart) {
                 if (currentPart == firstPart) {
-                    if (currentDigits != 4) {
+                    if (currentDigits < 4) {
                         throw Error.error(ErrorCode.X_22007);
                     }
                 } else {
@@ -2388,8 +2432,7 @@ public class Scanner {
         }
 
         fractionPrecision = currentDigits;
-        currentValue      *= DTIType.nanoScaleFactors[currentDigits];
-        currentValue = DTIType.normaliseFraction(currentValue,
+        currentValue = DTIType.normaliseFraction(currentValue, currentDigits,
                 decimalPrecision);
 
         return currentValue;
@@ -2589,8 +2632,8 @@ public class Scanner {
                 if (token.tokenType != Tokens.X_ENDPARSE) {
                     throw Error.error(errorCode);
                 }
+                break;
 
-            // fall through
             default :
         }
 
