@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2018, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,13 +39,12 @@ import org.hsqldb.lib.HashMappedList;
 import org.hsqldb.lib.HsqlArrayList;
 import org.hsqldb.lib.Iterator;
 import org.hsqldb.lib.OrderedHashSet;
-import org.hsqldb.map.ValuePool;
 
 /**
  * Collection of SQL schema objects of a specific type in a schema
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.4.1
+ * @version 2.5.0
  * @since 1.9.0
  */
 public class SchemaObjectSet {
@@ -259,7 +258,23 @@ public class SchemaObjectSet {
                     (SchemaObject) ((HashMappedList) map).get(i);
 
                 object.getName().rename(newName);
-                ((HashMappedList) map).setKey(i, name.name);
+                ((HashMappedList) map).setKey(i, newName.name);
+
+                break;
+            }
+            case SchemaObject.SPECIFIC_ROUTINE : {
+                int i = ((HashMappedList) map).getIndex(name.name);
+
+                if (i == -1) {
+                    int code = getGetErrorCode(name.type);
+
+                    throw Error.error(code, name.name);
+                }
+
+                Routine routine = (Routine) ((HashMappedList) map).get(i);
+
+                routine.getSpecificName().rename(newName);
+                ((HashMappedList) map).setKey(i, newName.name);
 
                 break;
             }
@@ -395,6 +410,9 @@ public class SchemaObjectSet {
             case SchemaObject.INDEX :
                 return Tokens.T_INDEX;
 
+            case SchemaObject.PERIOD :
+                return Tokens.T_PERIOD;
+
             default :
                 throw Error.runtimeError(ErrorCode.U_S0500, "SchemaObjectSet");
         }
@@ -424,8 +442,8 @@ public class SchemaObjectSet {
                     Routine routine = routineSchema.routines[i];
 
                     if (routine.dataImpact == Routine.NO_SQL
-                            || routine.dataImpact == Routine.CONTAINS_SQL ||
-                            routine.language == Routine.LANGUAGE_JAVA) {}
+                            || routine.dataImpact == Routine.CONTAINS_SQL
+                            || routine.language == Routine.LANGUAGE_JAVA) {}
                     else {
                         set.add(routine);
                     }
@@ -506,6 +524,7 @@ public class SchemaObjectSet {
                     // fall through
                     case SchemaObject.TYPE :
                     case SchemaObject.DOMAIN :
+                    case SchemaObject.COLLATION :
                     case SchemaObject.FUNCTION :
                     case SchemaObject.PROCEDURE :
                     case SchemaObject.SPECIFIC_ROUTINE :
