@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2018, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,23 +38,24 @@ import org.hsqldb.index.Index;
 import org.hsqldb.lib.HashMappedList;
 import org.hsqldb.lib.Iterator;
 import org.hsqldb.lib.OrderedHashSet;
-import org.hsqldb.lib.java.JavaSystem;
 import org.hsqldb.persist.DataFileCache;
 import org.hsqldb.persist.DataSpaceManager;
 import org.hsqldb.persist.HsqlDatabaseProperties;
 import org.hsqldb.persist.HsqlProperties;
 import org.hsqldb.persist.PersistentStore;
+import org.hsqldb.persist.ScriptLoader;
 import org.hsqldb.persist.TableSpaceManager;
 import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultMetaData;
 import org.hsqldb.rights.User;
 import org.hsqldb.scriptio.ScriptWriterText;
+import org.hsqldb.types.TimestampData;
 
 /**
  * Implementation of Statement for SQL commands.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.4.1
+ * @version 2.5.0
  * @since 1.9.0
  */
 public class StatementCommand extends Statement {
@@ -112,6 +113,12 @@ public class StatementCommand extends Statement {
 
                 break;
             }
+            case StatementTypes.LOAD_SCRIPT : {
+                group    = StatementTypes.X_HSQLDB_DATABASE_OPERATION;
+                isLogged = false;
+
+                break;
+            }
             case StatementTypes.CHECK_INDEX : {
                 statementReturnType = StatementTypes.RETURN_RESULT;
                 group = StatementTypes.X_HSQLDB_DATABASE_OPERATION;
@@ -163,6 +170,7 @@ public class StatementCommand extends Statement {
             case StatementTypes.SET_DATABASE_SQL :
             case StatementTypes.SET_DATABASE_DEFAULT_ISOLATION_LEVEL :
             case StatementTypes.SET_DATABASE_TRANSACTION_CONFLICT :
+            case StatementTypes.SET_DATABASE_TRANSACTION_INTERRUPT :
             case StatementTypes.SET_DATABASE_GC :
             case StatementTypes.SET_DATABASE_SQL_COLLATION :
             case StatementTypes.SET_DATABASE_FILES_BACKUP_INCREMENT :
@@ -329,10 +337,6 @@ public class StatementCommand extends Statement {
                         throw Error.error(ErrorCode.DATABASE_IS_READONLY);
                     }
 
-                    if (session.database.logger.isStoredFileAccess()) {
-                        throw Error.error(ErrorCode.ACCESS_IS_DENIED);
-                    }
-
                     session.database.logger.backup(path, script, blocking,
                                                    compressed, files);
 
@@ -862,14 +866,26 @@ public class StatementCommand extends Statement {
                     return Result.newErrorResult(e, sql);
                 }
             }
+            case StatementTypes.SET_DATABASE_TRANSACTION_INTERRUPT : {
+                try {
+                    boolean mode = ((Boolean) arguments[0]).booleanValue();
+
+                    session.checkAdmin();
+
+                    session.database.txInterruptRollback = mode;
+
+                    return Result.updateZeroResult;
+                } catch (HsqlException e) {
+                    return Result.newErrorResult(e, sql);
+                }
+            }
             case StatementTypes.SET_DATABASE_GC : {
                 try {
                     int count = ((Integer) arguments[0]).intValue();
 
                     session.checkAdmin();
 
-                    JavaSystem.gcFrequency = count;
-
+                    // a no-op from version 2.5.0
                     return Result.updateZeroResult;
                 } catch (HsqlException e) {
                     return Result.newErrorResult(e, sql);
@@ -935,8 +951,12 @@ public class StatementCommand extends Statement {
                 }
             }
             case StatementTypes.DATABASE_SCRIPT : {
-                ScriptWriterText dsw  = null;
-                String           name = (String) arguments[0];
+                ScriptWriterText dsw       = null;
+                String           name      = (String) arguments[0];
+                Integer          scope     = (Integer) arguments[1];
+                Integer          type      = (Integer) arguments[2];
+                HsqlName         tableName = (HsqlName) arguments[3];
+                TimestampData    timestamp = (TimestampData) arguments[4];
 
                 try {
                     session.checkAdmin();
@@ -947,7 +967,34 @@ public class StatementCommand extends Statement {
                         dsw = new ScriptWriterText(session.database, name,
                                                    true, true, true);
 
-                        dsw.writeAll();
+                        switch (type.intValue()) {
+
+                            case Tokens.ALL :
+                                dsw.writeAll();
+                                break;
+
+                            case Tokens.STRUCTURE :
+                                dsw.writeDDL();
+                                break;
+
+                            case Tokens.DATA :
+                                if (tableName == null) {
+                                    dsw.writeExistingData(false);
+                                } else {
+                                    Table table =
+                                        session.database.schemaManager
+                                            .getUserTable(tableName);
+
+                                    dsw.setIncludeTableInit(true);
+                                    dsw.writeTableData(table);
+                                }
+                                break;
+
+                            case Tokens.VERSIONING :
+                                dsw.writeVersioningData(timestamp);
+                                break;
+                        }
+
                         dsw.close();
 
                         return Result.updateZeroResult;
@@ -968,6 +1015,18 @@ public class StatementCommand extends Statement {
                     return Result.newErrorResult(e, sql);
                 }
             }
+            case StatementTypes.LOAD_SCRIPT : {
+                try {
+                    String  pathName     = (String) arguments[0];
+                    int     mode         = ((Integer) arguments[1]).intValue();
+                    Boolean isVersioning = (Boolean) arguments[2];
+
+                    return ScriptLoader.loadScriptData(
+                        session, pathName, mode, isVersioning.booleanValue());
+                } catch (HsqlException e) {
+                    return Result.newErrorResult(e, sql);
+                }
+            }
             case StatementTypes.CHECK_INDEX : {
                 try {
                     int type = ((Integer) arguments[1]).intValue();
@@ -1348,7 +1407,7 @@ public class StatementCommand extends Statement {
                         case Tokens.END :
                             session.database.txManager.resetSession(session,
                                     targetSession,
-                                    TransactionManager.resetSessionAbort);
+                                    TransactionManager.resetSessionStatement);
                             break;
                     }
                 } catch (HsqlException e) {
@@ -1366,10 +1425,11 @@ public class StatementCommand extends Statement {
     Result getTruncateResult(Session session) {
 
         try {
-            HsqlName name            = (HsqlName) arguments[0];
-            boolean  restartIdentity = (Boolean) arguments[1];
-            boolean  noCheck         = (Boolean) arguments[2];
-            Table[]  tables;
+            HsqlName      name            = (HsqlName) arguments[0];
+            boolean       restartIdentity = (Boolean) arguments[1];
+            boolean       noCheck         = (Boolean) arguments[2];
+            TimestampData timestamp       = (TimestampData) arguments[3];
+            Table[]       tables;
 
             if (name.type == SchemaObject.TABLE) {
                 Table table =
@@ -1379,6 +1439,14 @@ public class StatementCommand extends Statement {
 
                 session.getGrantee().checkDelete(table);
 
+                if (timestamp != null) {
+                    TablePeriodWorks works = new TablePeriodWorks(session,
+                        table);
+                    long count = works.removeOldRows(timestamp.getSeconds());
+
+                    return Result.newUpdateCountResult(0);
+                }
+
                 if (!noCheck) {
                     for (int i = 0; i < table.fkMainConstraints.length; i++) {
                         if (table.fkMainConstraints[i].getRef() != table) {
@@ -1467,7 +1535,11 @@ public class StatementCommand extends Statement {
         switch (type) {
 
             case StatementTypes.EXPLAIN_PLAN :
-                return ResultMetaData.newSingleColumnMetaData("OPERATION");
+                return ResultMetaData.newSingleColumnMetaData(Tokens.T_PLAN);
+
+            case StatementTypes.EXPLAIN_REFERENCES :
+                return ResultMetaData.newSingleColumnMetaData(
+                    Tokens.T_REFERENCES);
 
             case StatementTypes.DATABASE_SCRIPT :
                 if (statementReturnType == StatementTypes.RETURN_RESULT) {
@@ -1475,7 +1547,8 @@ public class StatementCommand extends Statement {
                         "STATEMENTS");
                 }
 
-            // fall through
+                return super.getResultMetaData();
+
             default :
                 return super.getResultMetaData();
         }
