@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,13 +40,14 @@ import org.hsqldb.navigator.RowSetNavigatorDataChangeMemory;
 import org.hsqldb.persist.PersistentStore;
 import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultConstants;
+import org.hsqldb.trigger.Trigger;
 import org.hsqldb.types.Type;
 
 /**
  * Implementation of Statement for INSERT statements.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.5.0
  * @since 1.9.0
  */
 public class StatementInsert extends StatementDML {
@@ -225,14 +226,11 @@ public class StatementInsert extends StatementDML {
             while (changeNavigator.next()) {
                 session.sessionData.startRowProcessing();
 
-                Row      row  = changeNavigator.getCurrentRow();
-                Object[] data = row.getData();
-                Object[] newData;
-
-                newData = getUpdatedData(session, targets, baseTable,
-                                         updateColumnMap, updateExpressions,
-                                         colTypes, data);
+                Row      row     = changeNavigator.getCurrentRow();
+                Object[] newData = row.getDataCopy();
 
+                getUpdatedData(session, targets, baseTable, updateColumnMap,
+                               updateExpressions, colTypes, newData);
                 changeNavigator.addUpdate(row, newData, updateColumnMap);
 
                 session.sessionContext.rownum++;
