@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,14 +31,13 @@
 
 package org.hsqldb;
 
-import org.hsqldb.persist.CachedObject;
 import org.hsqldb.persist.PersistentStore;
 
 /**
  * Manages rows involved in transactions
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.0
+ * @version 2.5.0
  * @since 2.0.0
  */
 public interface TransactionManager {
@@ -54,12 +53,12 @@ public interface TransactionManager {
     int ACTION_REF  = 2;
 
     //
-    int resetSessionResults  = 1;
-    int resetSessionTables   = 2;
-    int resetSessionResetAll = 3;
-    int resetSessionRollback = 4;
-    int resetSessionAbort    = 5;
-    int resetSessionClose    = 6;
+    int resetSessionResults   = 1;
+    int resetSessionTables    = 2;
+    int resetSessionResetAll  = 3;
+    int resetSessionRollback  = 4;
+    int resetSessionStatement = 5;
+    int resetSessionClose     = 6;
 
     long getGlobalChangeTimestamp();
 
@@ -73,6 +72,8 @@ public interface TransactionManager {
     void addInsertAction(Session session, Table table, PersistentStore store,
                          Row row, int[] changedColumns);
 
+    void addInsertAction(Session session, PersistentStore store, Row row);
+
     /**
      * add session to the end of queue when a transaction starts
      * (depending on isolation mode)
@@ -83,12 +84,6 @@ public interface TransactionManager {
 
     void beginTransaction(Session session);
 
-    // functional unit - accessibility of rows
-    boolean canRead(Session session, PersistentStore store, Row row, int mode,
-                    int[] colMap);
-
-    boolean canRead(Session session, PersistentStore store, long id, int mode);
-
     boolean commitTransaction(Session session);
 
     void completeActions(Session session);
@@ -113,24 +108,5 @@ public interface TransactionManager {
 
     void setTransactionControl(Session session, int mode);
 
-    /**
-     * store transaction info for a new row. called only
-     * for CACHED tables
-     */
-    void addTransactionInfo(CachedObject object);
-
-    /**
-     * add transaction info to a row just loaded from the cache. called only
-     * for CACHED tables
-     */
-    void setTransactionInfo(PersistentStore store, CachedObject object);
-
-    /**
-     * remove the transaction info
-     */
-    void removeTransactionInfo(CachedObject object);
-
-    void removeTransactionInfo(long id);
-
     void resetSession(Session session, Session targetSession, int mode);
 }
