@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2017, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,7 @@
 
 package org.hsqldb;
 
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
 
@@ -50,7 +51,7 @@ import org.hsqldb.lib.OrderedHashSet;
  * Shared code for TransactionManager classes
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.5
+ * @version 2.5.0
  * @since 2.0.0
  */
 class TransactionManagerCommon {
@@ -73,7 +74,7 @@ class TransactionManagerCommon {
     AtomicLong globalChangeTimestamp = new AtomicLong(1);
 
     //
-    long transactionCount = 0;
+    AtomicInteger transactionCount = new AtomicInteger();
 
     //
     HashMap           tableWriteLocks = new HashMap();
@@ -181,6 +182,17 @@ class TransactionManagerCommon {
         }
     }
 
+    void beginTransactionCommon(Session session) {
+
+        session.actionTimestamp      = getNextGlobalChangeTimestamp();
+        session.actionStartTimestamp = session.actionTimestamp;
+        session.transactionTimestamp = session.actionTimestamp;
+        session.isPreTransaction     = false;
+        session.isTransaction        = true;
+
+        transactionCount.incrementAndGet();
+    }
+
     void adjustLobUsage(Session session) {
 
         int  limit               = session.rowActionList.size();
@@ -545,10 +557,10 @@ class TransactionManagerCommon {
         final int waitingCount = session.waitingSessions.size();
 
         for (int i = 0; i < waitingCount; i++) {
-            Session current  = (Session) session.waitingSessions.get(i);
-            boolean testCode = false;
+            Session current     = (Session) session.waitingSessions.get(i);
+            boolean monitorCode = false;
 
-            if (testCode) {
+            if (monitorCode) {
                 if (!current.abortTransaction && current.tempSet.isEmpty()) {
 
                     // test code valid only for top level statements
@@ -582,10 +594,10 @@ class TransactionManagerCommon {
         final int waitingCount = session.tempSet.size();
 
         for (int i = 0; i < waitingCount; i++) {
-            Session current  = (Session) session.tempSet.get(i);
-            boolean testCode = false;
+            Session current     = (Session) session.tempSet.get(i);
+            boolean monitorCode = false;
 
-            if (testCode) {
+            if (monitorCode) {
                 if (!current.abortTransaction && current.tempSet.isEmpty()) {
 
                     // test code valid for top level statements
@@ -917,12 +929,20 @@ class TransactionManagerCommon {
             switch (mode) {
 
                 case TransactionManager.resetSessionResults :
+                    if (session != targetSession) {
+                        break;
+                    }
+
                     if (!targetSession.isInMidTransaction()) {
                         targetSession.sessionData.closeAllNavigators();
                     }
                     break;
 
                 case TransactionManager.resetSessionTables :
+                    if (session != targetSession) {
+                        break;
+                    }
+
                     if (!targetSession.isInMidTransaction()) {
                         targetSession.sessionData.persistentStoreCollection
                             .clearAllTables();
@@ -930,6 +950,10 @@ class TransactionManagerCommon {
                     break;
 
                 case TransactionManager.resetSessionResetAll :
+                    if (session != targetSession) {
+                        break;
+                    }
+
                     if (!targetSession.isInMidTransaction()) {
                         targetSession.resetSession();
                     }
@@ -943,17 +967,17 @@ class TransactionManagerCommon {
                     if (targetSession.isInMidTransaction()) {
                         prepareReset(targetSession);
 
-                        if (targetSession.latch.getCount() > 0) {
-                            targetSession.abortTransaction = true;
+                        targetSession.abortTransaction = true;
 
+                        if (targetSession.latch.getCount() > 0) {
                             targetSession.latch.setCount(0);
                         } else {
-                            targetSession.abortTransaction = true;
+                            targetSession.rollbackNoCheck(true);
                         }
                     }
                     break;
 
-                case TransactionManager.resetSessionAbort :
+                case TransactionManager.resetSessionStatement :
                     if (session == targetSession) {
                         return;
                     }
@@ -961,12 +985,10 @@ class TransactionManagerCommon {
                     if (targetSession.isInMidTransaction()) {
                         prepareReset(targetSession);
 
-                        if (targetSession.latch.getCount() > 0) {
-                            targetSession.abortAction = true;
+                        targetSession.abortAction = true;
 
+                        if (targetSession.latch.getCount() > 0) {
                             targetSession.latch.setCount(0);
-                        } else {
-                            targetSession.abortAction = true;
                         }
                     }
                     break;
