@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2017, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,18 +33,15 @@ package org.hsqldb;
 
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
-import org.hsqldb.lib.HashSet;
 import org.hsqldb.lib.HsqlDeque;
 import org.hsqldb.lib.LongDeque;
-import org.hsqldb.lib.LongKeyHashMap;
-import org.hsqldb.persist.CachedObject;
 import org.hsqldb.persist.PersistentStore;
 
 /**
  * Manages rows involved in transactions
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.5
+ * @version 2.5.0
  * @since 2.0.0
  */
 public class TransactionManagerMVCC extends TransactionManagerCommon
@@ -72,9 +69,8 @@ implements TransactionManager {
 
         super(db);
 
-        lobSession   = database.sessionManager.getSysLobSession();
-        rowActionMap = new LongKeyHashMap(8192);
-        txModel      = MVCC;
+        lobSession = database.sessionManager.getSysLobSession();
+        txModel    = MVCC;
     }
 
     public long getGlobalChangeTimestamp() {
@@ -123,7 +119,7 @@ implements TransactionManager {
             for (int i = 0; i < limit; i++) {
                 RowAction action = (RowAction) session.rowActionList.get(i);
 
-                if (!action.canCommit(session, session.actionSet)) {
+                if (!action.canCommit(session)) {
 
 //                System.out.println("commit conflicts " + session + " " + session.actionTimestamp);
                     return false;
@@ -166,7 +162,7 @@ implements TransactionManager {
             for (int i = 0; i < limit; i++) {
                 RowAction action = (RowAction) session.rowActionList.get(i);
 
-                if (!action.canCommit(session, session.actionSet)) {
+                if (!action.canCommit(session)) {
 
 //                  System.out.println("commit conflicts " + session + " " + session.actionTimestamp);
                     return false;
@@ -494,6 +490,20 @@ implements TransactionManager {
         }
     }
 
+    public void addInsertAction(Session session, PersistentStore store,
+                                Row row) {
+
+        RowAction action = row.rowAction;
+
+        if (action == null) {
+            throw Error.runtimeError(ErrorCode.GENERAL_ERROR,
+                                     "null insert action ");
+        }
+
+        store.indexRow(session, row);
+        session.rowActionList.add(action);
+    }
+
 // functional unit - accessibility of rows
     public boolean canRead(Session session, PersistentStore store, Row row,
                            int mode, int[] colMap) {
@@ -571,92 +581,6 @@ implements TransactionManager {
         return action.canRead(session, mode);
     }
 
-    public boolean canRead(Session session, PersistentStore store, long id,
-                           int mode) {
-
-        if (store.getTable().tableType == TableBase.TEMP_TABLE) {
-            return true;
-        }
-
-        RowAction action = (RowAction) rowActionMap.get(id);
-
-        if (action == null) {
-            return true;
-        }
-
-        return action.canRead(session, mode);
-    }
-
-    public void addTransactionInfo(CachedObject object) {
-
-        if (object.isMemory()) {
-            return;
-        }
-
-        Row row = (Row) object;
-
-        if (row.getTable().tableType == TableBase.CACHED_TABLE) {
-            RowAction action = (RowAction) rowActionMap.get(object.getPos());
-
-            if (action != null) {
-                HsqlException e = Error.error(ErrorCode.X_40501,
-                                              "TXManager - row exists");
-
-                this.database.logger.logSevereEvent("TXManager MVROWS", e);
-
-                throw e;
-            }
-
-            rowActionMap.put(object.getPos(), row.rowAction);
-        }
-    }
-
-    /**
-     * add transaction info to a row just loaded from the cache. called only
-     * for CACHED tables
-     */
-    public void setTransactionInfo(PersistentStore store,
-                                   CachedObject object) {
-
-        if (object.isMemory()) {
-            return;
-        }
-
-        Row row = (Row) object;
-
-        if (row.getTable().tableType == TableBase.CACHED_TABLE) {
-            RowAction rowact = (RowAction) rowActionMap.get(row.getPos());
-
-            row.rowAction = rowact;
-        }
-    }
-
-    /**
-     * remove the transaction info
-     */
-    public void removeTransactionInfo(CachedObject object) {
-        rowActionMap.remove(object.getPos());
-    }
-
-    public void removeTransactionInfo(long id) {
-
-        rowActionMap.getWriteLock().lock();
-
-        try {
-            RowAction action = (RowAction) rowActionMap.get(id);
-
-            synchronized (action) {
-
-                // remove only if not changed
-                if (action.type == RowActionBase.ACTION_NONE) {
-                    rowActionMap.remove(id);
-                }
-            }
-        } finally {
-            rowActionMap.getWriteLock().unlock();
-        }
-    }
-
     /**
      * add a list of actions to the end of queue
      */
@@ -715,15 +639,9 @@ implements TransactionManager {
 
         try {
             if (!session.isTransaction) {
-                session.actionTimestamp      = getNextGlobalChangeTimestamp();
-                session.transactionTimestamp = session.actionTimestamp;
-                session.isPreTransaction     = false;
-                session.isTransaction        = true;
-
+                beginTransactionCommon(session);
                 liveTransactionTimestamps.addLast(
                     session.transactionTimestamp);
-
-                transactionCount++;
             }
         } finally {
             writeLock.unlock();
@@ -782,20 +700,14 @@ implements TransactionManager {
         writeLock.lock();
 
         try {
-            session.actionTimestamp      = getNextGlobalChangeTimestamp();
-            session.actionStartTimestamp = session.actionTimestamp;
-
             if (session.isTransaction) {
-                return;
+                session.actionTimestamp      = getNextGlobalChangeTimestamp();
+                session.actionStartTimestamp = session.actionTimestamp;
+            } else {
+                beginTransactionCommon(session);
+                liveTransactionTimestamps.addLast(
+                    session.transactionTimestamp);
             }
-
-            session.transactionTimestamp = session.actionTimestamp;
-            session.isPreTransaction     = false;
-            session.isTransaction        = true;
-
-            liveTransactionTimestamps.addLast(session.transactionTimestamp);
-
-            transactionCount++;
         } finally {
             writeLock.unlock();
         }
@@ -805,52 +717,13 @@ implements TransactionManager {
                                    PersistentStore store, Row row,
                                    int[] colMap) {
 
-        RowAction action = null;
+        RowAction action = store.addDeleteActionToRow(session, row, colMap,
+            true);
 
-        synchronized (row) {
-            switch (table.tableType) {
+        if (table.tableType == TableBase.TEMP_TABLE) {
+            store.delete(session, row);
 
-                case TableBase.CACHED_TABLE : {
-                    rowActionMap.getWriteLock().lock();
-
-                    try {
-
-                        /* using rowActionMap as source */
-                        action = (RowAction) rowActionMap.get(row.getPos());
-
-                        if (action == null) {
-                            action = RowAction.addDeleteAction(session, table,
-                                                               row, colMap);
-
-                            if (action != null) {
-                                addTransactionInfo(row);
-                            }
-                        } else {
-                            row.rowAction = action;
-                            action = RowAction.addDeleteAction(session, table,
-                                                               row, colMap);
-                        }
-                    } finally {
-                        rowActionMap.getWriteLock().unlock();
-                    }
-
-                    break;
-                }
-                case TableBase.TEMP_TABLE : {
-                    action = RowAction.addDeleteAction(session, table, row,
-                                                       colMap);
-
-                    store.delete(session, row);
-
-                    row.rowAction = null;
-
-                    break;
-                }
-                case TableBase.MEMORY_TABLE :
-                default :
-                    action = RowAction.addDeleteAction(session, table, row,
-                                                       colMap);
-            }
+            row.rowAction = null;
         }
 
         return action;
@@ -868,8 +741,7 @@ implements TransactionManager {
         int  index     = liveTransactionTimestamps.indexOf(timestamp);
 
         if (index >= 0) {
-            transactionCount--;
-
+            transactionCount.decrementAndGet();
             liveTransactionTimestamps.remove(index);
             mergeExpiredTransactions(session);
         }
