@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2018, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -112,7 +112,7 @@ public class TriggerDefSQL extends TriggerDef {
 
     public String getSQL() {
 
-        StringBuffer sb = getSQLMain();
+        StringBuilder sb = getSQLMain();
 
         sb.append(routine.statement.getSQL());
 
