@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2017, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,118 +54,123 @@ import org.hsqldb.persist.PersistentStore;
  * DatabaseInformation implementations. <p>
  *
  * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
- * @version 2.3.5
+ * @version 2.5.0
  * @since 1.7.2
  */
 public class DatabaseInformation {
 
     // ids for system table names strictly in order of sysTableNames[]
-    protected static final int SYSTEM_BESTROWIDENTIFIER = 0;
-    protected static final int SYSTEM_COLUMNS           = 1;
-    protected static final int SYSTEM_CROSSREFERENCE    = 2;
-    protected static final int SYSTEM_INDEXINFO         = 3;
-    protected static final int SYSTEM_PRIMARYKEYS       = 4;
-    protected static final int SYSTEM_PROCEDURECOLUMNS  = 5;
-    protected static final int SYSTEM_PROCEDURES        = 6;
-    protected static final int SYSTEM_SCHEMAS           = 7;
-    protected static final int SYSTEM_TABLES            = 8;
-    protected static final int SYSTEM_TABLETYPES        = 9;
-    protected static final int SYSTEM_TYPEINFO          = 10;
-    protected static final int SYSTEM_UDTS              = 11;
-    protected static final int SYSTEM_USERS             = 12;    //-- ref in SqlFile only
-    protected static final int SYSTEM_VERSIONCOLUMNS    = 13;    //-- returns autogenerated columns
-    protected static final int SYSTEM_SEQUENCES = 14;            //-- same as SEQUENCES
+    static final int SYSTEM_BESTROWIDENTIFIER = 0;
+    static final int SYSTEM_COLUMNS           = 1;
+    static final int SYSTEM_CROSSREFERENCE    = 2;
+    static final int SYSTEM_INDEXINFO         = 3;
+    static final int SYSTEM_PRIMARYKEYS       = 4;
+    static final int SYSTEM_PROCEDURECOLUMNS  = 5;
+    static final int SYSTEM_PROCEDURES        = 6;
+    static final int SYSTEM_SCHEMAS           = 7;
+    static final int SYSTEM_TABLES            = 8;
+    static final int SYSTEM_TABLETYPES        = 9;
+    static final int SYSTEM_TYPEINFO          = 10;
+    static final int SYSTEM_UDTS              = 11;
+    static final int SYSTEM_USERS             = 12;    //-- ref in SqlFile only
+    static final int SYSTEM_VERSIONCOLUMNS    = 13;    //-- returns autogenerated columns
+    static final int SYSTEM_SEQUENCES = 14;            //-- same as SEQUENCES
 
     // HSQLDB-specific
-    protected static final int SYSTEM_CACHEINFO             = 15;
-    protected static final int SYSTEM_COLUMN_SEQUENCE_USAGE = 16;
-    protected static final int SYSTEM_COMMENTS              = 17;
-    protected static final int SYSTEM_CONNECTION_PROPERTIES = 18;
-    protected static final int SYSTEM_INDEXSTATS            = 19;
-    protected static final int SYSTEM_KEY_INDEX_USAGE       = 20;
-    protected static final int SYSTEM_PROPERTIES            = 21;
-    protected static final int SYSTEM_SESSIONINFO           = 22;
-    protected static final int SYSTEM_SESSIONS              = 23;
-    protected static final int SYSTEM_TABLESTATS            = 24;
-    protected static final int SYSTEM_TEXTTABLES            = 25;
-    protected static final int SYSTEM_SYNONYMS              = 26;
+    static final int SYSTEM_CACHEINFO             = 15;
+    static final int SYSTEM_COLUMN_SEQUENCE_USAGE = 16;
+    static final int SYSTEM_COMMENTS              = 17;
+    static final int SYSTEM_CONNECTION_PROPERTIES = 18;
+    static final int SYSTEM_INDEXSTATS            = 19;
+    static final int SYSTEM_KEY_INDEX_USAGE       = 20;
+    static final int SYSTEM_PROPERTIES            = 21;
+    static final int SYSTEM_SESSIONINFO           = 22;
+    static final int SYSTEM_SESSIONS              = 23;
+    static final int SYSTEM_TABLESTATS            = 24;
+    static final int SYSTEM_TEXTTABLES            = 25;
+    static final int SYSTEM_SYNONYMS              = 26;
 
     // SQL 200n tables
-    protected static final int ADMINISTRABLE_ROLE_AUTHORIZATIONS = 27;
-    protected static final int APPLICABLE_ROLES                  = 28;
-    protected static final int ASSERTIONS                        = 29;
-    protected static final int AUTHORIZATIONS                    = 30;
-    protected static final int CHARACTER_SETS                    = 31;
-    protected static final int CHECK_CONSTRAINT_ROUTINE_USAGE    = 32;
-    protected static final int CHECK_CONSTRAINTS                 = 33;
-    protected static final int COLLATIONS                        = 34;
-    protected static final int COLUMN_COLUMN_USAGE               = 35;
-    protected static final int COLUMN_DOMAIN_USAGE               = 36;
-    protected static final int COLUMN_PRIVILEGES                 = 37;
-    protected static final int COLUMN_UDT_USAGE                  = 38;
-    protected static final int COLUMNS                           = 39;
-    protected static final int CONSTRAINT_COLUMN_USAGE           = 40;
-    protected static final int CONSTRAINT_TABLE_USAGE            = 41;
-    protected static final int DATA_TYPE_PRIVILEGES              = 42;
-    protected static final int DOMAIN_CONSTRAINTS                = 43;
-    protected static final int DOMAINS                           = 44;
-    protected static final int ELEMENT_TYPES                     = 45;
-    protected static final int ENABLED_ROLES                     = 46;
-    protected static final int INFORMATION_SCHEMA_CATALOG_NAME   = 47;
-    protected static final int JAR_JAR_USAGE                     = 48;
-    protected static final int JARS                              = 49;
-    protected static final int KEY_COLUMN_USAGE                  = 50;
-    protected static final int METHOD_SPECIFICATIONS             = 51;
-    protected static final int MODULE_COLUMN_USAGE               = 52;
-    protected static final int MODULE_PRIVILEGES                 = 53;
-    protected static final int MODULE_TABLE_USAGE                = 54;
-    protected static final int MODULES                           = 55;
-    protected static final int PARAMETERS                        = 56;
-    protected static final int REFERENTIAL_CONSTRAINTS           = 57;
-    protected static final int ROLE_AUTHORIZATION_DESCRIPTORS    = 58;
-    protected static final int ROLE_COLUMN_GRANTS                = 59;
-    protected static final int ROLE_MODULE_GRANTS                = 60;
-    protected static final int ROLE_ROUTINE_GRANTS               = 61;
-    protected static final int ROLE_TABLE_GRANTS                 = 62;
-    protected static final int ROLE_UDT_GRANTS                   = 63;
-    protected static final int ROLE_USAGE_GRANTS                 = 64;
-    protected static final int ROUTINE_COLUMN_USAGE              = 65;
-    protected static final int ROUTINE_JAR_USAGE                 = 66;
-    protected static final int ROUTINE_PRIVILEGES                = 67;
-    protected static final int ROUTINE_ROUTINE_USAGE             = 68;
-    protected static final int ROUTINE_SEQUENCE_USAGE            = 69;
-    protected static final int ROUTINE_TABLE_USAGE               = 70;
-    protected static final int ROUTINES                          = 71;
-    protected static final int SCHEMATA                          = 72;
-    protected static final int SEQUENCES                         = 73;
-    protected static final int SQL_FEATURES                      = 74;
-    protected static final int SQL_IMPLEMENTATION_INFO           = 75;
-    protected static final int SQL_PACKAGES                      = 76;
-    protected static final int SQL_PARTS                         = 77;
-    protected static final int SQL_SIZING                        = 78;
-    protected static final int SQL_SIZING_PROFILES               = 79;
-    protected static final int TABLE_CONSTRAINTS                 = 80;
-    protected static final int TABLE_PRIVILEGES                  = 81;
-    protected static final int TABLES                            = 82;
-    protected static final int TRANSLATIONS                      = 83;
-    protected static final int TRIGGER_COLUMN_USAGE              = 84;
-    protected static final int TRIGGER_ROUTINE_USAGE             = 85;
-    protected static final int TRIGGER_SEQUENCE_USAGE            = 86;
-    protected static final int TRIGGER_TABLE_USAGE               = 87;
-    protected static final int TRIGGERED_UPDATE_COLUMNS          = 88;
-    protected static final int TRIGGERS                          = 89;
-    protected static final int TYPE_JAR_USAGE                    = 90;
-    protected static final int UDT_PRIVILEGES                    = 91;
-    protected static final int USAGE_PRIVILEGES                  = 92;
-    protected static final int USER_DEFINED_TYPES                = 93;
-    protected static final int VIEW_COLUMN_USAGE                 = 94;
-    protected static final int VIEW_ROUTINE_USAGE                = 95;
-    protected static final int VIEW_TABLE_USAGE                  = 96;
-    protected static final int VIEWS                             = 97;
-
+    static final int ADMINISTRABLE_ROLE_AUTHORIZATIONS = 27;
+    static final int APPLICABLE_ROLES                  = 28;
+    static final int ASSERTIONS                        = 29;
+    static final int AUTHORIZATIONS                    = 30;
+    static final int CHARACTER_SETS                    = 31;
+    static final int CHECK_CONSTRAINT_ROUTINE_USAGE    = 32;
+    static final int CHECK_CONSTRAINTS                 = 33;
+    static final int COLLATIONS                        = 34;
+    static final int COLUMN_COLUMN_USAGE               = 35;
+    static final int COLUMN_DOMAIN_USAGE               = 36;
+    static final int COLUMN_PRIVILEGES                 = 37;
+    static final int COLUMN_UDT_USAGE                  = 38;
+    static final int COLUMNS                           = 39;
+    static final int CONSTRAINT_COLUMN_USAGE           = 40;
+    static final int CONSTRAINT_PERIOD_USAGE           = 41;
+    static final int CONSTRAINT_TABLE_USAGE            = 42;
+    static final int DATA_TYPE_PRIVILEGES              = 43;
+    static final int DOMAIN_CONSTRAINTS                = 44;
+    static final int DOMAINS                           = 45;
+    static final int ELEMENT_TYPES                     = 46;
+    static final int ENABLED_ROLES                     = 47;
+    static final int INFORMATION_SCHEMA_CATALOG_NAME   = 48;
+    static final int JAR_JAR_USAGE                     = 49;
+    static final int JARS                              = 50;
+    static final int KEY_COLUMN_USAGE                  = 51;
+    static final int KEY_PERIOD_USAGE                  = 52;
+    static final int METHOD_SPECIFICATIONS             = 53;
+    static final int MODULE_COLUMN_USAGE               = 54;
+    static final int MODULE_PRIVILEGES                 = 55;
+    static final int MODULE_TABLE_USAGE                = 56;
+    static final int MODULES                           = 57;
+    static final int PARAMETERS                        = 58;
+    static final int PERIODS                           = 59;
+    static final int REFERENTIAL_CONSTRAINTS           = 60;
+    static final int ROLE_AUTHORIZATION_DESCRIPTORS    = 61;
+    static final int ROLE_COLUMN_GRANTS                = 62;
+    static final int ROLE_MODULE_GRANTS                = 63;
+    static final int ROLE_ROUTINE_GRANTS               = 64;
+    static final int ROLE_TABLE_GRANTS                 = 65;
+    static final int ROLE_UDT_GRANTS                   = 66;
+    static final int ROLE_USAGE_GRANTS                 = 67;
+    static final int ROUTINE_COLUMN_USAGE              = 68;
+    static final int ROUTINE_JAR_USAGE                 = 69;
+    static final int ROUTINE_PERIOD_USAGE              = 70;
+    static final int ROUTINE_PRIVILEGES                = 71;
+    static final int ROUTINE_ROUTINE_USAGE             = 72;
+    static final int ROUTINE_SEQUENCE_USAGE            = 73;
+    static final int ROUTINE_TABLE_USAGE               = 74;
+    static final int ROUTINES                          = 75;
+    static final int SCHEMATA                          = 76;
+    static final int SEQUENCES                         = 77;
+    static final int SQL_FEATURES                      = 78;
+    static final int SQL_IMPLEMENTATION_INFO           = 79;
+    static final int SQL_PACKAGES                      = 80;
+    static final int SQL_PARTS                         = 81;
+    static final int SQL_SIZING                        = 82;
+    static final int SQL_SIZING_PROFILES               = 83;
+    static final int TABLE_CONSTRAINTS                 = 84;
+    static final int TABLE_PRIVILEGES                  = 85;
+    static final int TABLES                            = 86;
+    static final int TRANSLATIONS                      = 87;
+    static final int TRIGGER_COLUMN_USAGE              = 88;
+    static final int TRIGGER_PERIOD_USAGE              = 89;
+    static final int TRIGGER_ROUTINE_USAGE             = 90;
+    static final int TRIGGER_SEQUENCE_USAGE            = 91;
+    static final int TRIGGER_TABLE_USAGE               = 92;
+    static final int TRIGGERED_UPDATE_COLUMNS          = 93;
+    static final int TRIGGERS                          = 94;
+    static final int TYPE_JAR_USAGE                    = 95;
+    static final int UDT_PRIVILEGES                    = 96;
+    static final int USAGE_PRIVILEGES                  = 97;
+    static final int USER_DEFINED_TYPES                = 98;
+    static final int VIEW_COLUMN_USAGE                 = 99;
+    static final int VIEW_PERIOD_USAGE                 = 100;
+    static final int VIEW_ROUTINE_USAGE                = 101;
+    static final int VIEW_TABLE_USAGE                  = 102;
+    static final int VIEWS                             = 103;
 
     /** system table names strictly in order of their ids */
-    protected static final String[] sysTableNames = {
+    static final String[] sysTableNames = {
         "SYSTEM_BESTROWIDENTIFIER",                              //
         "SYSTEM_COLUMNS",                                        //
         "SYSTEM_CROSSREFERENCE",                                 //
@@ -211,6 +216,7 @@ public class DatabaseInformation {
         "COLUMN_UDT_USAGE",                                      //
         "COLUMNS",                                               //
         "CONSTRAINT_COLUMN_USAGE",                               //
+        "CONSTRAINT_PERIOD_USAGE",                               //
         "CONSTRAINT_TABLE_USAGE",                                //
         "DATA_TYPE_PRIVILEGES",                                  //
         "DOMAIN_CONSTRAINTS",                                    //
@@ -221,12 +227,14 @@ public class DatabaseInformation {
         "JAR_JAR_USAGE",                                         //
         "JARS",                                                  //
         "KEY_COLUMN_USAGE",                                      //
+        "KEY_PERIOD_USAGE",                                      //
         "METHOD_SPECIFICATIONS",                                 //
         "MODULE_COLUMN_USAGE",                                   //
         "MODULE_PRIVILEGES",                                     //
         "MODULE_TABLE_USAGE",                                    //
         "MODULES",                                               //
         "PARAMETERS",                                            //
+        "PERIODS",                                               //
         "REFERENTIAL_CONSTRAINTS",                               //
         "ROLE_AUTHORIZATION_DESCRIPTORS",                        //
         "ROLE_COLUMN_GRANTS",                                    //
@@ -237,6 +245,7 @@ public class DatabaseInformation {
         "ROLE_USAGE_GRANTS",                                     //
         "ROUTINE_COLUMN_USAGE",                                  //
         "ROUTINE_JAR_USAGE",                                     //
+        "ROUTINE_PERIOD_USAGE",                                  //
         "ROUTINE_PRIVILEGES",                                    //
         "ROUTINE_ROUTINE_USAGE",                                 //
         "ROUTINE_SEQUENCE_USAGE",                                //
@@ -255,6 +264,7 @@ public class DatabaseInformation {
         "TABLES",                                                //
         "TRANSLATIONS",                                          //
         "TRIGGER_COLUMN_USAGE",                                  //
+        "TRIGGER_PERIOD_USAGE",                                  //
         "TRIGGER_ROUTINE_USAGE",                                 //
         "TRIGGER_SEQUENCE_USAGE",                                //
         "TRIGGER_TABLE_USAGE",                                   //
@@ -265,17 +275,18 @@ public class DatabaseInformation {
         "USAGE_PRIVILEGES",                                      //
         "USER_DEFINED_TYPES",                                    //
         "VIEW_COLUMN_USAGE",                                     //
+        "VIEW_PERIOD_USAGE",                                     //
         "VIEW_ROUTINE_USAGE",                                    //
         "VIEW_TABLE_USAGE",                                      //
         "VIEWS",                                                 //
     };
 
     /** Map: table name => table id */
-    protected static final IntValueHashMap sysTableNamesMap;
+    static final IntValueHashMap sysTableNamesMap;
 
     static {
         synchronized (DatabaseInformation.class) {
-            sysTableNamesMap = new IntValueHashMap(101);
+            sysTableNamesMap = new IntValueHashMap(107);
 
             for (int i = 0; i < sysTableNames.length; i++) {
                 sysTableNamesMap.put(sysTableNames[i], i);
@@ -288,7 +299,7 @@ public class DatabaseInformation {
     }
 
     /** Database for which to produce tables */
-    protected final Database database;
+    final Database database;
 
     /**
      * state flag -- if true, tables are to be produced with content, else
@@ -296,7 +307,7 @@ public class DatabaseInformation {
      * database startup where user views reference system tables and faster
      * system table structural reflection for table metadata.
      */
-    protected boolean withContent = false;
+    boolean withContent = false;
 
     /**
      * Factory method returns the fullest system table producer
@@ -307,25 +318,22 @@ public class DatabaseInformation {
      * @return the fullest system table producer
      *      implementation available
      */
-    public static DatabaseInformation newDatabaseInformation(
-            Database db) {
+    public static DatabaseInformation newDatabaseInformation(Database db) {
 
-        Class c = null;
+        Class<?> c = DatabaseInformation.class;
 
         try {
             c = Class.forName("org.hsqldb.dbinfo.DatabaseInformationFull");
         } catch (Exception e) {
             try {
                 c = Class.forName("org.hsqldb.dbinfo.DatabaseInformationMain");
-            } catch (Exception e2) {
-                c = DatabaseInformation.class;
-            }
+            } catch (Exception e2) {}
         }
 
         try {
-            Class[]     ctorParmTypes = new Class[]{ Database.class };
-            Object[]    ctorParms     = new Object[]{ db };
-            Constructor ctor = c.getDeclaredConstructor(ctorParmTypes);
+            Class<?>[]     ctorParmTypes = new Class[]{ Database.class };
+            Object[]       ctorParms     = new Object[]{ db };
+            Constructor<?> ctor = c.getDeclaredConstructor(ctorParmTypes);
 
             return (DatabaseInformation) ctor.newInstance(ctorParms);
         } catch (Exception e) {
@@ -374,7 +382,7 @@ public class DatabaseInformation {
      * Sets the store for the given session, populates the store if necessary.
      */
     public void setStore(Session session, Table table,
-                                      PersistentStore store) {}
+                         PersistentStore store) {}
 
     /**
      * Switches this table producer between producing empty (surrogate)
