@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2017, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,14 +36,10 @@ import java.sql.Connection;
 import java.sql.DatabaseMetaData;
 import java.sql.ResultSet;
 import java.sql.SQLException;
-import java.sql.SQLWarning;
-import java.sql.Statement;
-
-//#ifdef JAVA6
 import java.sql.SQLFeatureNotSupportedException;
+import java.sql.SQLWarning;
 import java.sql.SQLTimeoutException;
-
-//#endif JAVA6
+import java.sql.Statement;
 
 import org.hsqldb.HsqlException;
 import org.hsqldb.StatementTypes;
@@ -52,7 +48,7 @@ import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultConstants;
 import org.hsqldb.result.ResultProperties;
 
-/* $Id: JDBCStatement.java 5740 2017-04-02 15:57:38Z fredt $ */
+/* $Id: JDBCStatement.java 5968 2019-04-27 12:55:27Z fredt $ */
 
 // fredt@users   20020320 - patch 1.7.0 - JDBC 2 support and error trapping
 //
@@ -112,17 +108,9 @@ import org.hsqldb.result.ResultProperties;
  * @see JDBCConnection#createStatement
  * @see JDBCResultSet
  */
-//#ifdef JAVA6
 public class JDBCStatement extends JDBCStatementBase implements Statement,
         java.sql.Wrapper {
 
-//#else
-/*
-public class JDBCStatement extends JDBCStatementBase implements Statement {
-*/
-
-//#endif JAVA6
-
     /**
      * <!-- start generic documentation -->
      * Executes the given SQL statement, which returns a single
@@ -1621,7 +1609,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @since JDK 1.6 Build 81, HSQLDB 2.0
      */
 
-//#ifdef JAVA6
     public synchronized void setPoolable(
             boolean poolable) throws SQLException {
 
@@ -1630,8 +1617,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
         this.poolable = poolable;
     }
 
-//#endif JAVA6
-
     /**
      * Returns a  value indicating whether the <code>Statement</code>
      * is poolable or not.
@@ -1645,7 +1630,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * <p>
      * @see #setPoolable(boolean) setPoolable(boolean)
      */
-//#ifdef JAVA6
     public synchronized boolean isPoolable() throws SQLException {
 
         checkClosed();
@@ -1653,7 +1637,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
         return this.poolable;
     }
 
-//#endif JAVA6
     // ------------------- java.sql.Wrapper implementation ---------------------
 
     /**
@@ -1673,7 +1656,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @throws java.sql.SQLException If no object found that implements the interface
      * @since JDK 1.6, HSQLDB 2.0
      */
-//#ifdef JAVA6
     @SuppressWarnings("unchecked")
     public <T>T unwrap(Class<T> iface) throws java.sql.SQLException {
 
@@ -1684,8 +1666,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
         throw JDBCUtil.invalidArgument("iface: " + iface);
     }
 
-//#endif JAVA6
-
     /**
      * Returns true if this either implements the interface argument or is directly or indirectly a wrapper
      * for an object that does. Returns false otherwise. If this implements the interface then return true,
@@ -1701,14 +1681,11 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * for an object with the given interface.
      * @since JDK 1.6, HSQLDB 2.0
      */
-//#ifdef JAVA6
     public boolean isWrapperFor(
             java.lang.Class<?> iface) throws java.sql.SQLException {
         return (iface != null && iface.isAssignableFrom(this.getClass()));
     }
 
-//#endif JAVA6
-
     //--------------------------JDBC 4.2 -----------------------------
 
     /**
@@ -1729,11 +1706,9 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @since 1.8
      */
 //#ifdef JAVA8
-/*
     public long getLargeUpdateCount() throws SQLException {
         return super.getUpdateCount();
     }
-*/
 
 //#endif JAVA8
     /**
@@ -1756,14 +1731,12 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @since 1.8
      */
 //#ifdef JAVA8
-/*
     public void setLargeMaxRows(long max) throws SQLException {
 
         int maxRows = max > Integer.MAX_VALUE ? Integer.MAX_VALUE :
                                                 (int) max;
         setMaxRows(maxRows);
     }
-*/
 //#endif JAVA8
 
     /**
@@ -1786,11 +1759,9 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @since 1.8
      */
 //#ifdef JAVA8
-/*
     public long getLargeMaxRows() throws SQLException {
         return maxRows;
     }
-*/
 //#endif JAVA8
 
     /**
@@ -1849,7 +1820,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @since 1.8
      */
 //#ifdef JAVA8
-/*
     public long[] executeLargeBatch() throws SQLException {
         int[] updateCounts = executeBatch();
 
@@ -1861,7 +1831,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
 
         return longCounts;
     }
-*/
 //#endif JAVA8
 
     /**
@@ -1896,11 +1865,9 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @since 1.8
      */
 //#ifdef JAVA8
-/*
     public long executeLargeUpdate(String sql) throws SQLException {
         return executeUpdate(sql);
     }
-*/
 //#endif JAVA8
 
     /**
@@ -1947,12 +1914,10 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @since 1.8
      */
 //#ifdef JAVA8
-/*
     public long executeLargeUpdate(String sql, int autoGeneratedKeys)
             throws SQLException {
         return executeUpdate(sql, autoGeneratedKeys);
     }
-*/
 //#endif JAVA8
 
     /**
@@ -1996,11 +1961,9 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @since 1.8
      */
 //#ifdef JAVA8
-/*
-    public long executeLargeUpdate(String sql, int columnIndexes[]) throws SQLException {
+    public long executeLargeUpdate(String sql, int[] columnIndexes) throws SQLException {
         return executeUpdate(sql, columnIndexes);
     }
-*/
 //#endif JAVA8
 
     /**
@@ -2043,12 +2006,10 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @since 1.8
      */
 //#ifdef JAVA8
-/*
-    public long executeLargeUpdate(String sql, String columnNames[])
+    public long executeLargeUpdate(String sql, String[] columnNames)
             throws SQLException {
         return executeUpdate(sql, columnNames);
     }
-*/
 //#endif JAVA8
     // -------------------- Internal Implementation ----------------------------
 
