@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2017, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -77,11 +77,12 @@ import javax.transaction.xa.Xid;
  */
 public class JDBCXID implements Xid {
 
-    int formatID;
+    int    formatID;
     byte[] txID;
     byte[] txBranch;
+
     //
-    int hash;
+    int     hash;
     boolean hashComputed;
 
     public int getFormatId() {
@@ -99,18 +100,20 @@ public class JDBCXID implements Xid {
     public JDBCXID(int formatID, byte[] txID, byte[] txBranch) {
 
         this.formatID = formatID;
-        this.txID = txID;
+        this.txID     = txID;
         this.txBranch = txBranch;
     }
 
     public int hashCode() {
+
         if (!hashComputed) {
-            hash = 7;
-            hash = 83 * hash + this.formatID;
-            hash = 83 * hash + Arrays.hashCode(this.txID);
-            hash = 83 * hash + Arrays.hashCode(this.txBranch);
+            hash         = 7;
+            hash         = 83 * hash + this.formatID;
+            hash         = 83 * hash + Arrays.hashCode(this.txID);
+            hash         = 83 * hash + Arrays.hashCode(this.txBranch);
             hashComputed = true;
         }
+
         return hash;
     }
 
@@ -120,8 +123,8 @@ public class JDBCXID implements Xid {
             Xid o = (Xid) other;
 
             return formatID == o.getFormatId()
-                    && Arrays.equals(txID, o.getGlobalTransactionId())
-                    && Arrays.equals(txBranch, o.getBranchQualifier());
+                   && Arrays.equals(txID, o.getGlobalTransactionId())
+                   && Arrays.equals(txBranch, o.getBranchQualifier());
         }
 
         return false;
@@ -130,50 +133,69 @@ public class JDBCXID implements Xid {
     // inherit doc
     public String toString() {
 
-        StringBuffer sb = new StringBuffer(512);
+        StringBuilder sb = new StringBuilder(512);
+
         //
         sb.append("formatId=").append(getFormatId());
+
         //
         sb.append(" globalTransactionId(").append(txID.length).append(")={0x");
+
         for (int i = 0; i < txID.length; i++) {
             final int hexVal = txID[i] & 0xFF;
+
             if (hexVal < 0x10) {
                 sb.append("0").append(Integer.toHexString(txID[i] & 0xFF));
             }
+
             sb.append(Integer.toHexString(txID[i] & 0xFF));
         }
+
         //
-        sb.append("} branchQualifier(").append(txBranch.length).append("))={0x");
+        sb.append("} branchQualifier(").append(txBranch.length).append(
+            "))={0x");
+
         for (int i = 0; i < txBranch.length; i++) {
             final int hexVal = txBranch[i] & 0xFF;
+
             if (hexVal < 0x10) {
                 sb.append("0");
             }
+
             sb.append(Integer.toHexString(txBranch[i] & 0xFF));
         }
+
         sb.append("}");
+
         //
         return sb.toString();
     }
 
-    private static byte[] s_localIp = null;
-    private static int s_txnSequenceNumber = 0;
+    private static byte[] s_localIp           = null;
+    private static int    s_txnSequenceNumber = 0;
+
     //
     private static final int UXID_FORMAT_ID = 0xFEED;
 
     private static int nextTxnSequenceNumber() {
-         s_txnSequenceNumber++;
-        return  s_txnSequenceNumber;
+
+        s_txnSequenceNumber++;
+
+        return s_txnSequenceNumber;
     }
 
     private static byte[] getLocalIp() {
+
         if (null == s_localIp) {
             try {
                 s_localIp = Inet4Address.getLocalHost().getAddress();
             } catch (Exception ex) {
-                s_localIp = new byte[]{0x7F, 0x00, 0x00, 0x01};
+                s_localIp = new byte[] {
+                    0x7F, 0x00, 0x00, 0x01
+                };
             }
         }
+
         return s_localIp;
     }
 
@@ -192,15 +214,18 @@ public class JDBCXID implements Xid {
      * @return a randomly generated JDBCXID
      */
     public static Xid getUniqueXid(final int threadId) {
+
         final Random random = new Random(System.currentTimeMillis());
+
         //
         int txnSequenceNumberValue = nextTxnSequenceNumber();
-        int threadIdValue = threadId;
-        int randomValue = random.nextInt();
+        int threadIdValue          = threadId;
+        int randomValue            = random.nextInt();
+
         //
         byte[] globalTransactionId = new byte[MAXGTRIDSIZE];
-        byte[] branchQualifier = new byte[MAXBQUALSIZE];
-        byte[] localIp = getLocalIp();
+        byte[] branchQualifier     = new byte[MAXBQUALSIZE];
+        byte[] localIp             = getLocalIp();
 
         System.arraycopy(localIp, 0, globalTransactionId, 0, 4);
         System.arraycopy(localIp, 0, branchQualifier, 0, 4);
@@ -209,17 +234,19 @@ public class JDBCXID implements Xid {
         // Bytes 8 ->11 - thread id.
         // Bytes 12->15 - random.
         for (int i = 0; i <= 3; i++) {
-            globalTransactionId[i + 4] = (byte) (txnSequenceNumberValue % 0x100);
+            globalTransactionId[i + 4] = (byte) (txnSequenceNumberValue
+                                                 % 0x100);
             branchQualifier[i + 4] = (byte) (txnSequenceNumberValue % 0x100);
-            txnSequenceNumberValue >>= 8;
-            globalTransactionId[i + 8] = (byte) (threadIdValue % 0x100);
-            branchQualifier[i + 8] = (byte) (threadIdValue % 0x100);
-            threadIdValue >>= 8;
+            txnSequenceNumberValue      >>= 8;
+            globalTransactionId[i + 8]  = (byte) (threadIdValue % 0x100);
+            branchQualifier[i + 8]      = (byte) (threadIdValue % 0x100);
+            threadIdValue               >>= 8;
             globalTransactionId[i + 12] = (byte) (randomValue % 0x100);
-            branchQualifier[i + 12] = (byte) (randomValue % 0x100);
-            randomValue >>= 8;
+            branchQualifier[i + 12]     = (byte) (randomValue % 0x100);
+            randomValue                 >>= 8;
         }
 
-        return new JDBCXID(UXID_FORMAT_ID, globalTransactionId, branchQualifier);
+        return new JDBCXID(UXID_FORMAT_ID, globalTransactionId,
+                           branchQualifier);
     }
 }
