@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2018, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,7 +39,7 @@ import java.util.concurrent.locks.AbstractQueuedSynchronizer;
  * as down.
  *
  * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
- * @version 2.4.0
+ * @version 2.5.0
  */
 public class CountUpDownLatch {
 
@@ -481,7 +481,7 @@ public class CountUpDownLatch {
 
                 if (amount > Integer.MAX_VALUE - c) {
                     throw new ArithmeticException(
-                        String.format("integer overflow: %d + $d", c, amount));
+                        String.format("integer overflow: %d", amount));
                 }
 
                 int nextc = c + amount;
