@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,6 @@ package org.hsqldb.lib;
 
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.UnsupportedEncodingException;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.Enumeration;
@@ -46,8 +45,9 @@ import java.util.ResourceBundle;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
+import org.hsqldb.lib.java.JavaSystem;
 
-/* $Id: RefCapablePropertyResourceBundle.java 5553 2016-03-16 23:17:40Z fredt $ */
+/* $Id: RefCapablePropertyResourceBundle.java 5968 2019-04-27 12:55:27Z fredt $ */
 
 /**
  * Just like PropertyResourceBundle, except keys mapped to nothing in the
@@ -199,7 +199,7 @@ public class RefCapablePropertyResourceBundle {
         String s = getString(key);
         Matcher matcher = sysPropVarPattern.matcher(s);
         int previousEnd = 0;
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         String varName, varValue;
         String condlVal;  // Conditional : value
         while (matcher.find()) {
@@ -243,9 +243,9 @@ public class RefCapablePropertyResourceBundle {
      * Note that %{\d} numbers are 1-based, so we lok for subs[x-1].
      */
     public String posSubst(String s, String[] subs, int behavior) {
-        Matcher matcher = posPattern.matcher(s);
-        int previousEnd = 0;
-        StringBuffer sb = new StringBuffer();
+        Matcher matcher  = posPattern.matcher(s);
+        int previousEnd  = 0;
+        StringBuilder sb = new StringBuilder();
         String varValue;
         int varIndex;
         String condlVal;  // Conditional : value
@@ -392,7 +392,7 @@ public class RefCapablePropertyResourceBundle {
                 + ".text";
         // System.err.println("Seeking " + filePath);
         InputStream is = (InputStream) AccessController.doPrivileged(
-            new PrivilegedAction() {
+            new PrivilegedAction<InputStream>() {
 
             public InputStream run() {
                 return loader.getResourceAsStream(filePath);
@@ -421,17 +421,18 @@ public class RefCapablePropertyResourceBundle {
         try {
             try {
                 ba = new byte[inputStream.available()];
-            } catch (RuntimeException re) {
+            } catch (IOException ioe) {
+                throw new MissingResourceException(
+                    "Failed to read in value for key '" + key + "': " + ioe,
+                    RefCapablePropertyResourceBundle.class.getName(), key);
+            } catch (Throwable re) {
                 throw new MissingResourceException(
                     "Resource is too big to read in '" + key + "' value in one "
                     + "gulp.\nPlease run the program with more RAM "
                     + "(try Java -Xm* switches).: " + re,
                     RefCapablePropertyResourceBundle.class.getName(), key);
-            } catch (IOException ioe) {
-                throw new MissingResourceException(
-                    "Failed to read in value for key '" + key + "': " + ioe,
-                    RefCapablePropertyResourceBundle.class.getName(), key);
             }
+
             try {
                 while (bytesread < ba.length &&
                         (retval = inputStream.read(
@@ -457,11 +458,10 @@ public class RefCapablePropertyResourceBundle {
                       + " bytes for key '" + key + "'",
                     RefCapablePropertyResourceBundle.class.getName(), key);
         }
+
         try {
-            return new String(ba, "ISO-8859-1");
-        } catch (UnsupportedEncodingException uee) {
-            throw new RuntimeException(uee);
-        } catch (RuntimeException re) {
+            return new String(ba, JavaSystem.CS_ISO_8859_1);
+        } catch (Throwable re) {
             throw new MissingResourceException(
                 "Value for key '" + key + "' too big to convert to String.  "
                 + "Please run the program with more RAM "
