@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2018, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -53,11 +53,11 @@ import org.hsqldb.rowio.RowOutputInterface;
  * Implementation of RowSetNavigator for result sets.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.5
+ * @version 2.5.0
  * @since 1.9.0
  */
 public class RowSetNavigatorData extends RowSetNavigator
-implements Comparator {
+implements Comparator<Object[]> {
 
     public static final Object[][] emptyTable = new Object[0][];
 
@@ -81,7 +81,7 @@ implements Comparator {
     Index idIndex;
 
     //
-    TreeMap        rowMap;
+    TreeMap<Object[], Object[]>        rowMap;
     LongKeyHashMap idMap;
 
     RowSetNavigatorData(Session session) {
@@ -100,7 +100,7 @@ implements Comparator {
 
         if (select.isGrouped) {
             mainIndex = select.groupIndex;
-            rowMap    = new TreeMap(this);
+            rowMap    = new TreeMap<Object[], Object[]>(this);
         }
 
         if (select.idIndex != null) {
@@ -602,7 +602,7 @@ implements Comparator {
             return simpleAggregateData;
         }
 
-        return (Object[]) rowMap.get(data);
+        return rowMap.get(data);
     }
 
     boolean containsRow(Object[] data) {
@@ -708,8 +708,7 @@ implements Comparator {
         }
     }
 
-    public int compare(Object a, Object b) {
-        return mainIndex.compareRow((Session) session, (Object[]) a,
-                                    (Object[]) b);
+    public int compare(Object[] a, Object[] b) {
+        return mainIndex.compareRow((Session) session, a, b);
     }
 }
