@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2018, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -117,7 +117,7 @@ public class RowSetNavigatorDataTable extends RowSetNavigatorData {
         mainIndex          = table.getPrimaryIndex();
         fullIndex          = table.getFullIndex(session);
         store              = table.getRowStore(session);
-        this.size          = (int) mainIndex.size(session, store);
+        this.size          = (int) store.elementCount();
 
         reset();
     }
@@ -255,7 +255,7 @@ public class RowSetNavigatorDataTable extends RowSetNavigatorData {
             iterator.release();
         }
 
-        iterator = mainIndex.firstRow((Session) session, store, 0, null);
+        iterator = mainIndex.firstRow((Session) session, store, null, 0, null);
     }
 
     public void release() {
