@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2018, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,7 +65,6 @@ public class HsqlProperties {
     protected Properties    stringProps;
     protected int[]         errorCodes = ValuePool.emptyIntArray;
     protected String[]      errorKeys  = ValuePool.emptyStringArray;
-    protected boolean       resource   = false;
     protected FileAccess    fa;
     protected HashMap       metaData;
 
@@ -606,7 +605,7 @@ public class HsqlProperties {
 
     public String toString() {
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
 
         sb.append('{');
 
