@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,6 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.LineNumberReader;
-import java.io.UnsupportedEncodingException;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.concurrent.locks.Lock;
@@ -60,6 +59,7 @@ import org.hsqldb.lib.ArrayUtil;
 import org.hsqldb.lib.HashMappedList;
 import org.hsqldb.lib.HsqlByteArrayInputStream;
 import org.hsqldb.lib.LineGroupReader;
+import org.hsqldb.lib.java.JavaSystem;
 import org.hsqldb.map.ValuePool;
 import org.hsqldb.navigator.RowSetNavigator;
 import org.hsqldb.result.Result;
@@ -75,7 +75,7 @@ import org.hsqldb.types.Types;
 
 /**
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.5.0
  * @since 1.9.0
  */
 public class LobManager {
@@ -258,24 +258,17 @@ public class LobManager {
 
         sysLobSession = database.sessionManager.getSysLobSession();
 
-        InputStream fis = (InputStream) AccessController.doPrivileged(
-            new PrivilegedAction() {
-
-            public InputStream run() {
-                return getClass().getResourceAsStream(resourceFileName);
-            }
-        });
-        InputStreamReader reader = null;
-
-        try {
-            reader = new InputStreamReader(fis, "ISO-8859-1");
-        } catch (UnsupportedEncodingException e) {
-            reader = new InputStreamReader(fis);
-        }
+        InputStream fis = AccessController.doPrivileged(
+                new PrivilegedAction<InputStream>() {
 
+                    public InputStream run() {
+                        return getClass().getResourceAsStream(resourceFileName);
+                    }
+                });
+        InputStreamReader reader = new InputStreamReader(fis, JavaSystem.CS_ISO_8859_1);
         LineNumberReader lineReader = new LineNumberReader(reader);
-        LineGroupReader  lg = new LineGroupReader(lineReader, starters);
-        HashMappedList   map        = lg.getAsMap();
+        LineGroupReader lg = new LineGroupReader(lineReader, starters);
+        HashMappedList map = lg.getAsMap();
 
         lg.close();
 
@@ -373,21 +366,28 @@ public class LobManager {
             dataBuffer = new byte[largeBufferBlockSize];
         }
 
-        if (database.getType() == DatabaseType.DB_RES) {
-            lobStore = new LobStoreInJar(database, lobBlockSize);
-        } else if (database.getType() == DatabaseType.DB_FILE) {
-            lobStore = new LobStoreRAFile(database, lobBlockSize);
+        switch (database.getType()) {
 
-            if (!database.isFilesReadOnly()) {
+            case DB_FILE :
+                lobStore = new LobStoreRAFile(database, lobBlockSize);
+
+                if (!database.isFilesReadOnly()) {
+                    byteBuffer = new byte[lobBlockSize];
+
+                    initialiseLobSpace();
+                }
+                break;
+
+            case DB_RES :
+                lobStore = new LobStoreInJar(database, lobBlockSize);
+                break;
+
+            case DB_MEM :
+                lobStore   = new LobStoreMem(lobBlockSize);
                 byteBuffer = new byte[lobBlockSize];
 
                 initialiseLobSpace();
-            }
-        } else {
-            lobStore   = new LobStoreMem(lobBlockSize);
-            byteBuffer = new byte[lobBlockSize];
-
-            initialiseLobSpace();
+                break;
         }
     }
 
@@ -606,7 +606,7 @@ public class LobManager {
             ResultMetaData meta   = deleteUnusedLobs.getParametersMetaData();
             Object[]       params = new Object[meta.getColumnCount()];
 
-            params[0] = new Long(firstLobID);
+            params[0] = Long.valueOf(firstLobID);
 
             Result result =
                 sysLobSession.executeCompiledStatement(deleteUnusedLobs,
@@ -700,14 +700,14 @@ public class LobManager {
         }
     }
 
-    public int compare(BlobData a, byte[] b) {
+    public int compare(long lobId, byte[] b) {
 
         writeLock.lock();
 
         try {
-            Object[] data    = getLobHeader(a.getId());
+            Object[] data    = getLobHeader(lobId);
             long     aLength = ((Long) data[LOB_IDS.LOB_LENGTH]).longValue();
-            int[][] aAddresses = getBlockAddresses(a.getId(), 0,
+            int[][] aAddresses = getBlockAddresses(lobId, 0,
                                                    Integer.MAX_VALUE);
             int aIndex  = 0;
             int bOffset = 0;
@@ -795,14 +795,14 @@ public class LobManager {
     }
 
     /** @todo - implement as compareText() */
-    public int compare(Collation collation, ClobData a, String b) {
+    public int compare(Collation collation, long lobId, String b) {
 
         writeLock.lock();
 
         try {
-            Object[] data    = getLobHeader(a.getId());
+            Object[] data    = getLobHeader(lobId);
             long     aLength = ((Long) data[LOB_IDS.LOB_LENGTH]).longValue();
-            int[][] aAddresses = getBlockAddresses(a.getId(), 0,
+            int[][] aAddresses = getBlockAddresses(lobId, 0,
                                                    Integer.MAX_VALUE);
             int aIndex  = 0;
             int bOffset = 0;
@@ -2064,7 +2064,7 @@ public class LobManager {
             getPartBytesCompressedInBuffer(aID, bParts[i], false);
 
             int result = ArrayUtil.compare(aPartBytes, 0, aPartLength,
-                                           byteBuffer, bPartLength);
+                                           byteBuffer, 0, bPartLength);
 
             if (result != 0) {
                 return result;
