@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2017, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,27 +55,29 @@ import org.hsqldb.navigator.RowIterator;
 import org.hsqldb.rowio.RowInputInterface;
 import org.hsqldb.types.LobData;
 import org.hsqldb.types.Type;
+import org.hsqldb.lib.LongKeyHashMap;
 
 /*
  * Base implementation of PersistentStore for different table types.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.5
+ * @version 2.5.0
  * @since 1.9.0
  */
 public abstract class RowStoreAVL implements PersistentStore {
 
-    Database          database;
-    TableSpaceManager tableSpace;
-    Index[]           indexList    = Index.emptyArray;
-    CachedObject[]    accessorList = CachedObject.emptyArray;
-    TableBase         table;
-    long              baseElementCount;
-    AtomicLong        elementCount = new AtomicLong();
-    long              storageSize;
-    boolean[]         nullsList;
-    double[][]        searchCost;
-    boolean           isSchemaStore;
+    Database              database;
+    TableSpaceManager     tableSpace;
+    Index[]               indexList    = Index.emptyArray;
+    CachedObject[]        accessorList = CachedObject.emptyArray;
+    TableBase             table;
+    long                  baseElementCount;
+    AtomicLong            elementCount = new AtomicLong();
+    long                  storageSize;
+    boolean[]             nullsList;
+    double[][]            searchCost;
+    boolean               isSchemaStore;
+    public LongKeyHashMap rowActionMap;
 
     //
     ReadWriteLock lock;
@@ -119,10 +121,6 @@ public abstract class RowStoreAVL implements PersistentStore {
 
     public abstract CachedObject get(CachedObject object, boolean keep);
 
-    public CachedObject getRow(long key, boolean[] usedColumnCheck) {
-        return get(key, false);
-    }
-
     public int compare(Session session, long key) {
         throw Error.runtimeError(ErrorCode.U_S0500, "RowStoreAVL");
     }
@@ -170,7 +168,13 @@ public abstract class RowStoreAVL implements PersistentStore {
 
     public abstract void commitPersistence(CachedObject object);
 
-    public abstract void postCommitAction(Session session, RowAction action);
+    public RowAction addDeleteActionToRow(Session session, Row row,
+                                          int[] colMap, boolean isMV) {
+
+        synchronized (row) {
+            return RowAction.addDeleteAction(session, table, row, colMap);
+        }
+    }
 
     public abstract DataFileCache getCache();
 
@@ -277,6 +281,10 @@ public abstract class RowStoreAVL implements PersistentStore {
                 indexList[i].delete(session, this, row);
             }
 
+            if (rowActionMap != null) {
+                rowActionMap.remove(row.getPos());
+            }
+
             remove(row);
 
             throw e;
@@ -290,6 +298,10 @@ public abstract class RowStoreAVL implements PersistentStore {
                 indexList[i].delete(session, this, row);
             }
 
+            if (rowActionMap != null) {
+                rowActionMap.remove(row.getPos());
+            }
+
             // do not remove as there may be still be reference
             throw Error.error(ErrorCode.GENERAL_ERROR, t);
         } finally {
@@ -297,6 +309,14 @@ public abstract class RowStoreAVL implements PersistentStore {
         }
     }
 
+    public abstract void commitRow(Session session, Row row, int changeAction,
+                                   int txModel);
+
+    public abstract void rollbackRow(Session session, Row row,
+                                     int changeAction, int txModel);
+
+    public abstract void postCommitAction(Session session, RowAction action);
+
     //
     public final void indexRows(Session session) {
 
@@ -323,7 +343,7 @@ public abstract class RowStoreAVL implements PersistentStore {
         }
     }
 
-    public final RowIterator rowIterator() {
+    public RowIterator rowIterator() {
 
         Index index = indexList[0];
 
@@ -421,15 +441,15 @@ public abstract class RowStoreAVL implements PersistentStore {
         return indexList;
     }
 
-    public synchronized double searchCost(Session session, Index index,
-                                          int count, int opType) {
+    public double searchCost(Session session, Index index, int count,
+                             int opType) {
 
         if (count == 0) {
             return elementCount.get();
         }
 
         if (opType != OpTypes.EQUAL) {
-            return elementCount.get() / 2.0;
+            return ((double) elementCount.get()) / 2;
         }
 
         if (index.isUnique() && count == index.getColumnCount()) {
@@ -438,16 +458,18 @@ public abstract class RowStoreAVL implements PersistentStore {
 
         int position = index.getPosition();
 
-        if (searchCost == null || searchCost.length != indexList.length) {
-            searchCost = new double[indexList.length][];
-        }
+        synchronized (this) {
+            if (searchCost == null || searchCost.length != indexList.length) {
+                searchCost = new double[indexList.length][];
+            }
 
-        if (searchCost[position] == null) {
-            searchCost[position] = indexList[position].searchCost(session,
-                    this);
-        }
+            if (searchCost[position] == null) {
+                searchCost[position] = indexList[position].searchCost(session,
+                        this);
+            }
 
-        return searchCost[index.getPosition()][count - 1];
+            return searchCost[index.getPosition()][count - 1];
+        }
     }
 
     public long elementCount() {
@@ -469,24 +491,26 @@ public abstract class RowStoreAVL implements PersistentStore {
 
     public long elementCount(Session session) {
 
-        if (session != null) {
-            Index index = this.indexList[0];
+        if (session == null) {
+            return elementCount();
+        }
 
-            if (session.database.txManager.isMVRows()) {
-                switch (table.getTableType()) {
+        Index index = this.indexList[0];
 
-                    case TableBase.MEMORY_TABLE :
-                    case TableBase.CACHED_TABLE :
-                    case TableBase.TEXT_TABLE :
-                        readLock();
+        if (session.database.txManager.isMVRows()) {
+            switch (table.getTableType()) {
 
-                        try {
-                            return index.getNodeCount(session, this);
-                        } finally {
-                            readUnlock();
-                        }
-                    default :
-                }
+                case TableBase.MEMORY_TABLE :
+                case TableBase.CACHED_TABLE :
+                case TableBase.TEXT_TABLE :
+                    readLock();
+
+                    try {
+                        return index.getNodeCount(session, this);
+                    } finally {
+                        readUnlock();
+                    }
+                default :
             }
         }
 
@@ -509,52 +533,49 @@ public abstract class RowStoreAVL implements PersistentStore {
 
     /**
      * Moves the data from an old store to new after changes to table
-     * The colindex argument is the index of the column that was
-     * added or removed. The adjust argument is {-1 | 0 | +1}
+     * The colIndex argument is the indexes of the columns that were
+     * removed, type modified or added. The adjust argument is {-1 | 0 | +1}.
      */
     public final void moveData(Session session, PersistentStore other,
-                               int colindex, int adjust) {
-
-        Type   oldtype  = null;
-        Type   newtype  = null;
-        Object colvalue = null;
-
-        if (adjust >= 0 && colindex != -1) {
-            ColumnSchema column = ((Table) table).getColumn(colindex);
-
-            colvalue = column.getDefaultValue(session);
-            newtype  = table.getColumnTypes()[colindex];
-        }
-
-        if (adjust <= 0 && colindex != -1) {
-            oldtype = other.getTable().getColumnTypes()[colindex];
-        }
+                               int[] colIndex, int adjust) {
 
         try {
             Table       table = (Table) this.table;
             RowIterator it    = other.rowIterator();
 
             while (it.next()) {
-                Row      row      = it.getCurrentRow();
-                Object[] olddata  = row.getData();
-                Object[] data     = table.getEmptyRowData();
-                Object   oldvalue = null;
+                Row      row     = it.getCurrentRow();
+                Object[] olddata = row.getData();
+                Object[] data    = table.getEmptyRowData();
 
-                if (adjust == 0 && colindex != -1) {
-                    oldvalue = olddata[colindex];
-                    colvalue = newtype.convertToType(session, oldvalue,
-                                                     oldtype);
+                ArrayUtil.copyAdjustArray(olddata, data, colIndex, adjust);
+
+                for (int i = 0; i < colIndex.length; i++) {
+                    if (adjust == 0) {
+                        Type oldtype =
+                            other.getTable().getColumnTypes()[colIndex[i]];
+                        Type newtype =
+                            this.table.getColumnTypes()[colIndex[i]];
+                        Object oldvalue = olddata[colIndex[i]];
+                        Object value = newtype.convertToType(session,
+                                                             oldvalue,
+                                                             oldtype);
+
+                        data[colIndex[i]] = value;
+
+                        table.systemSetIdentityColumn(session, data);
+                    } else if (adjust > 0) {
+                        ColumnSchema column = table.getColumn(colIndex[i]);
+                        Object value =
+                            table.getColumnDefaultOrGeneratedValue(session,
+                                column, data);
+
+                        data[colIndex[i]] = value;
+
+                        table.systemSetIdentityColumn(session, data);
+                    } else {}
                 }
 
-                ArrayUtil.copyAdjustArray(olddata, data, colvalue, colindex,
-                                          adjust);
-                table.systemSetIdentityColumn(session, data);
-
-                if (table.hasGeneratedColumn()) {
-                    ((Table) table).setGeneratedColumns(session, data);
-                }
-
-                table.enforceTypeLimits(session, data);
                 table.enforceRowConstraints(session, data);
 
                 // get object without RowAction
@@ -567,30 +588,43 @@ public abstract class RowStoreAVL implements PersistentStore {
                 return;
             }
 
-            if (oldtype != null && oldtype.isLobType()) {
-                it = other.rowIterator();
+            if (colIndex.length == 0) {
+                return;
+            }
 
-                while (it.next()) {
-                    Row      row      = it.getCurrentRow();
-                    Object[] olddata  = row.getData();
-                    LobData  oldvalue = (LobData) olddata[colindex];
+            if (adjust <= 0) {
+                Type type = other.getTable().getColumnTypes()[colIndex[0]];
 
-                    if (oldvalue != null) {
-                        session.sessionData.adjustLobUsageCount(oldvalue, -1);
+                if (type != null && type.isLobType()) {
+                    it = other.rowIterator();
+
+                    while (it.next()) {
+                        Row      row      = it.getCurrentRow();
+                        Object[] olddata  = row.getData();
+                        LobData  oldvalue = (LobData) olddata[colIndex[0]];
+
+                        if (oldvalue != null) {
+                            session.sessionData.adjustLobUsageCount(oldvalue,
+                                    -1);
+                        }
                     }
                 }
             }
 
-            if (newtype != null && newtype.isLobType()) {
-                it = rowIterator();
+            if (adjust >= 0) {
+                Type type = this.table.getColumnTypes()[colIndex[0]];
 
-                while (it.next()) {
-                    Row      row   = it.getCurrentRow();
-                    Object[] data  = row.getData();
-                    LobData  value = (LobData) data[colindex];
+                if (type != null && type.isLobType()) {
+                    it = rowIterator();
 
-                    if (value != null) {
-                        session.sessionData.adjustLobUsageCount(value, +1);
+                    while (it.next()) {
+                        Row      row   = it.getCurrentRow();
+                        Object[] data  = row.getData();
+                        LobData  value = (LobData) data[colIndex[0]];
+
+                        if (value != null) {
+                            session.sessionData.adjustLobUsageCount(value, +1);
+                        }
                     }
                 }
             }
