@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,7 @@
 
 package org.hsqldb.rights;
 
+import org.hsqldb.ExpressionLogical;
 import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.SchemaObject;
 import org.hsqldb.Table;
@@ -46,23 +47,24 @@ import org.hsqldb.lib.OrderedHashSet;
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
  *
- * @version 2.3.3
+ * @version 2.5.0
  * @since 1.9.0
  */
 public final class Right {
 
-    boolean        isFull;
-    boolean        isFullSelect;
-    boolean        isFullInsert;
-    boolean        isFullUpdate;
-    boolean        isFullReferences;
-    boolean        isFullTrigger;
-    boolean        isFullDelete;
-    OrderedHashSet selectColumnSet;
-    OrderedHashSet insertColumnSet;
-    OrderedHashSet updateColumnSet;
-    OrderedHashSet referencesColumnSet;
-    OrderedHashSet triggerColumnSet;
+    boolean           isFull;
+    boolean           isFullSelect;
+    boolean           isFullInsert;
+    boolean           isFullUpdate;
+    boolean           isFullReferences;
+    boolean           isFullTrigger;
+    boolean           isFullDelete;
+    OrderedHashSet    selectColumnSet;
+    OrderedHashSet    insertColumnSet;
+    OrderedHashSet    updateColumnSet;
+    OrderedHashSet    referencesColumnSet;
+    OrderedHashSet    triggerColumnSet;
+    ExpressionLogical filterExpression;
 
     //
     Right   grantableRights;
@@ -70,16 +72,12 @@ public final class Right {
     Grantee grantee;
 
     //
-    public static final OrderedHashSet emptySet      = new OrderedHashSet();
-    public static final Right          fullRights    = new Right();
-    public static final Right          noRights      = new Right();
-    static final OrderedHashSet        fullRightsSet = new OrderedHashSet();
+    public static final OrderedHashSet emptySet   = new OrderedHashSet();
+    public static final Right          fullRights = new Right(true);
+    public static final Right          noRights   = new Right();
 
     static {
         fullRights.grantor = GranteeManager.systemAuthorisation;
-        fullRights.isFull  = true;
-
-        fullRightsSet.add(fullRights);
     }
 
     public static final String[] privilegeNames = {
@@ -96,15 +94,8 @@ public final class Right {
         this.isFull = false;
     }
 
-    Right(Table table) {
-
-        isFull              = false;
-        isFullDelete        = true;
-        selectColumnSet     = table.getColumnNameSet();
-        insertColumnSet     = table.getColumnNameSet();
-        updateColumnSet     = table.getColumnNameSet();
-        referencesColumnSet = table.getColumnNameSet();
-        triggerColumnSet    = table.getColumnNameSet();
+    public Right(boolean full) {
+        isFull = full;
     }
 
     public boolean isFull() {
@@ -130,9 +121,20 @@ public final class Right {
 
         right.add(this);
 
+        if (filterExpression != null) {
+            right.filterExpression = filterExpression;
+        }
+
         return right;
     }
 
+    public void setFilterExpression(ExpressionLogical filter) {
+
+        if (filter != null) {
+            this.filterExpression = filter;
+        }
+    }
+
     /**
      * Supports column level GRANT
      */
@@ -754,12 +756,16 @@ public final class Right {
         return result;
     }
 
+    public ExpressionLogical getFilterExpression() {
+        return filterExpression;
+    }
+
     /**
      * supports column level GRANT
      */
     String getTableRightsSQL(Table table) {
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
 
         if (isFull) {
             return Tokens.T_ALL;
@@ -817,7 +823,7 @@ public final class Right {
     }
 
     private static void getColumnList(Table t, OrderedHashSet set,
-                                      StringBuffer buf) {
+                                      StringBuilder buf) {
 
         int       count        = 0;
         boolean[] colCheckList = t.getNewColumnCheckList();
@@ -983,7 +989,7 @@ public final class Right {
                 triggerColumnSet = set;
                 break;
 
-            default:
+            default :
         }
     }
 
