@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2018, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -46,7 +46,10 @@ import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.lib.LineReader;
 import org.hsqldb.lib.StringConverter;
+import org.hsqldb.lib.java.JavaSystem;
 import org.hsqldb.map.ValuePool;
+import org.hsqldb.persist.RowInsertInterface;
+import org.hsqldb.persist.RowInsertSimple;
 import org.hsqldb.result.Result;
 import org.hsqldb.rowio.RowInputTextLog;
 import org.hsqldb.types.Type;
@@ -57,7 +60,7 @@ import org.hsqldb.types.Type;
  * corresponds to ScriptWriterText.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- *  @version 2.4.1
+ *  @version 2.5.0
  *  @since 1.7.2
  */
 public class ScriptReaderText extends ScriptReaderBase {
@@ -92,10 +95,30 @@ public class ScriptReaderText extends ScriptReaderBase {
             tempStream = bufferedStream;
         }
 
-        dataStreamIn = new LineReader(tempStream, ScriptWriterText.ISO_8859_1);
+        dataStreamIn = new LineReader(tempStream, JavaSystem.CS_ISO_8859_1);
         rowIn = new RowInputTextLog(db.databaseProperties.isVersion18());
     }
 
+    public void readAll(Session session) {
+
+        int insertErrorMode;
+
+        if (database.recoveryMode == 0) {
+            errorLogger     = new RowInsertSimple.DefaultErrorHandler();
+            insertErrorMode = RowInsertInterface.modes.discardOnError;
+        } else {
+            errorLogger = new RowInsertSimple.InsertErrorHandler(database,
+                    fileNamePath);
+            insertErrorMode = RowInsertInterface.modes.continueOnError;
+        }
+
+        readDDL(session);
+
+        inserter = new RowInsertSimple(session, errorLogger, insertErrorMode);
+
+        readExistingData(session);
+    }
+
     protected void readDDL(Session session) {
 
         for (;;) {
@@ -109,7 +132,8 @@ public class ScriptReaderText extends ScriptReaderBase {
                     break;
                 }
 
-                if (rowIn.getStatementType() == INSERT_STATEMENT) {
+                if (rowIn.getStatementType()
+                        == StatementLineTypes.INSERT_STATEMENT) {
                     isInsert = true;
 
                     break;
@@ -133,15 +157,12 @@ public class ScriptReaderText extends ScriptReaderBase {
                 } else if (cs.getType() == StatementTypes.CREATE_ROUTINE) {
 
                     // ignore legacy references
-                    if (result.getMainString().indexOf("org.hsqldb.Library")
-                            > -1) {
+                    if (result.getMainString().contains(
+                            "org.hsqldb.Library")) {
                         continue;
                     }
                 }
 
-                database.logger.logWarningEvent(result.getMainString(),
-                                                result.getException());
-
                 HsqlException e = getError(result.getException(), lineCount);
 
                 handleError(e);
@@ -149,58 +170,75 @@ public class ScriptReaderText extends ScriptReaderBase {
         }
     }
 
-    protected void readExistingData(Session session) {
+    public void readExistingData(Session session) {
 
         String tablename = null;
 
-        for (;;) {
-            try {
-                boolean hasRow = false;
+        try {
+            for (;;) {
+                try {
+                    boolean hasRow = false;
 
-                if (isInsert) {
-                    isInsert = false;
-                    hasRow   = true;
-                } else {
-                    hasRow = readLoggedStatement(session);
-                }
-
-                if (!hasRow) {
-                    break;
-                }
-
-                if (statementType == SET_SCHEMA_STATEMENT) {
-                    session.setSchema(currentSchema);
-
-                    tablename = null;
-
-                    continue;
-                } else if (statementType == INSERT_STATEMENT) {
-                    if (!rowIn.getTableName().equals(tablename)) {
-                        tablename = rowIn.getTableName();
-
-                        String schema = session.getSchemaName(currentSchema);
-
-                        currentTable =
-                            database.schemaManager.getUserTable(tablename,
-                                schema);
-                        currentStore =
-                            database.persistentStoreCollection.getStore(
-                                currentTable);
+                    if (isInsert) {
+                        isInsert = false;
+                        hasRow   = true;
+                    } else {
+                        hasRow = readLoggedStatement(session);
                     }
 
-                    currentTable.insertFromScript(session, currentStore,
-                                                  rowData);
-                } else {
-                    HsqlException e = Error.error(ErrorCode.GENERAL_ERROR,
-                                                  statement);
+                    if (!hasRow) {
+                        inserter.finishTable();
 
-                    throw e;
+                        break;
+                    }
+
+                    switch (statementType) {
+
+                        case StatementLineTypes.SET_SCHEMA_STATEMENT : {
+                            session.setSchema(currentSchema);
+
+                            tablename = null;
+
+                            break;
+                        }
+                        case StatementLineTypes.INSERT_STATEMENT : {
+                            if (!rowIn.getTableName().equals(tablename)) {
+                                inserter.finishTable();
+
+                                tablename = rowIn.getTableName();
+
+                                String schema =
+                                    session.getSchemaName(currentSchema);
+
+                                currentTable =
+                                    database.schemaManager.getUserTable(
+                                        tablename, schema);
+                                currentStore =
+                                    database.persistentStoreCollection
+                                        .getStore(currentTable);
+                            }
+
+                            inserter.insert(currentTable, currentStore,
+                                            rowData);
+
+                            break;
+                        }
+                        default : {
+                            HsqlException e =
+                                Error.error(ErrorCode.GENERAL_ERROR,
+                                            statement);
+
+                            throw e;
+                        }
+                    }
+                } catch (Throwable t) {
+                    HsqlException e = getError(t, lineCount);
+
+                    handleError(e);
                 }
-            } catch (Throwable t) {
-                HsqlException e = getError(t, lineCount);
-
-                handleError(e);
             }
+        } finally {
+            inserter.close();
         }
     }
 
@@ -238,7 +276,7 @@ public class ScriptReaderText extends ScriptReaderBase {
             sessionNumber  = Integer.parseInt(statement.substring(3, endid));
             statement      = statement.substring(endid + 2);
             sessionChanged = true;
-            statementType  = SESSION_ID;
+            statementType  = StatementLineTypes.SESSION_ID;
 
             return;
         }
@@ -249,22 +287,22 @@ public class ScriptReaderText extends ScriptReaderBase {
 
         statementType = rowIn.getStatementType();
 
-        if (statementType == ANY_STATEMENT) {
-            rowData      = null;
-            currentTable = null;
+        switch (statementType) {
 
-            return;
-        } else if (statementType == COMMIT_STATEMENT) {
-            rowData      = null;
-            currentTable = null;
+            case StatementLineTypes.ANY_STATEMENT :
+            case StatementLineTypes.COMMIT_STATEMENT : {
+                rowData      = null;
+                currentTable = null;
 
-            return;
-        } else if (statementType == SET_SCHEMA_STATEMENT) {
-            rowData       = null;
-            currentTable  = null;
-            currentSchema = rowIn.getSchemaName();
+                return;
+            }
+            case StatementLineTypes.SET_SCHEMA_STATEMENT : {
+                rowData       = null;
+                currentTable  = null;
+                currentSchema = rowIn.getSchemaName();
 
-            return;
+                return;
+            }
         }
 
         String name   = rowIn.getTableName();
@@ -276,7 +314,7 @@ public class ScriptReaderText extends ScriptReaderBase {
 
         Type[] colTypes;
 
-        if (statementType == INSERT_STATEMENT) {
+        if (statementType == StatementLineTypes.INSERT_STATEMENT) {
             colTypes = currentTable.getColumnTypes();
         } else if (currentTable.hasPrimaryKey()) {
             colTypes = currentTable.getPrimaryKeyTypes();
@@ -308,8 +346,8 @@ public class ScriptReaderText extends ScriptReaderBase {
         } catch (Exception e) {}
 
         try {
-            if (scrwriter != null) {
-                scrwriter.close();
+            if (errorLogger != null) {
+                errorLogger.close();
             }
 
             database.recoveryMode = 0;
@@ -335,29 +373,13 @@ public class ScriptReaderText extends ScriptReaderBase {
 
     private void handleError(HsqlException e) {
 
-        database.logger.logSevereEvent("bad line in script file " + lineCount,
-                                       e);
+        database.logger.logSevereEvent(e.getMessage(), e);
 
         if (database.recoveryMode == 0) {
             throw e;
         }
 
-        openScriptWriter();
-
-        try {
-            scrwriter.writeLogStatement(null, rawStatement);
-        } catch (Throwable t) {}
-    }
-
-    private void openScriptWriter() {
-
-        if (scrwriter == null) {
-            String timestamp =
-                database.logger.fileDateFormat.format(new java.util.Date());
-            String name = fileNamePath + "." + timestamp + ".reject";
-
-            scrwriter = new ScriptWriterText(database, name, false, false,
-                                             true);
-        }
+        // todo - write line number etc
+        errorLogger.writeLogStatement(lineCount, rawStatement);
     }
 }
