@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -362,9 +362,6 @@ public class TestSchemaParse extends TestCase {
         execSQL("DROP TABLE bigtbl", 0);
         execSQL("DROP SEQUENCE tstseq", 0);
         execSQL("SET FILES LOG SIZE 5", 0);
-
-        // Following syntax is now obsolete.
-        execSQL("SET PROPERTY \"hsqldb.first_identity\" 4", 0);
         execSQL("UPDATE tsttbl SET vc = 'eleven' WHERE i = 1", 1);
         execSQL(
             "ALTER TABLE constrainedtbl ADD CONSTRAINT con1 CHECK (i6 > 4)",
