@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2014, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -147,7 +147,7 @@ public class TestViewAsterisks extends TestBase {
     private void createView(String viewName, String[] columnList,
                             String viewStatement) throws SQLException {
 
-        StringBuffer colList = new StringBuffer();
+        StringBuilder colList = new StringBuilder();
 
         if (columnList != null) {
             colList.append(" (");
@@ -268,7 +268,7 @@ public class TestViewAsterisks extends TestBase {
         String actualTranslatedStatement = getViewStatement(viewName);
 
         if (!actualTranslatedStatement.equals(expectedTranslatedStatement)) {
-            StringBuffer message = new StringBuffer();
+            StringBuilder message = new StringBuilder();
 
             message.append(viewName).append(
                 "'s statement not translated as expected\n");
@@ -390,13 +390,13 @@ public class TestViewAsterisks extends TestBase {
 
         // ................................................................
         checkViewTranslationAndContent(
-            "C1", null, "SELECT * AS \"a2\" FROM ABC",
-            "SELECT PUBLIC.ABC.ID,PUBLIC.ABC.A,PUBLIC.ABC.B,PUBLIC.ABC.C AS \"a2\" FROM PUBLIC.ABC",
+            "C1", null, "SELECT * FROM ABC",
+            "SELECT PUBLIC.ABC.ID,PUBLIC.ABC.A,PUBLIC.ABC.B,PUBLIC.ABC.C FROM PUBLIC.ABC",
             new Object[][] {
             new Object[] {
-                new Integer(1), "a", "b", "c"
+                Integer.valueOf(1), "a", "b", "c"
             }, new Object[] {
-                new Integer(2), "d", "e", "f"
+                Integer.valueOf(2), "d", "e", "f"
             }
         });
 
@@ -406,9 +406,9 @@ public class TestViewAsterisks extends TestBase {
             "SELECT B AS \"b2\", PUBLIC.ABC.ID,PUBLIC.ABC.A,PUBLIC.ABC.B,PUBLIC.ABC.C  FROM PUBLIC.ABC",
             new Object[][] {
             new Object[] {
-                "b", new Integer(1), "a", "b", "c"
+                "b", Integer.valueOf(1), "a", "b", "c"
             }, new Object[] {
-                "e", new Integer(2), "d", "e", "f"
+                "e", Integer.valueOf(2), "d", "e", "f"
             }
         });
     }
@@ -424,13 +424,13 @@ public class TestViewAsterisks extends TestBase {
             "SELECT PUBLIC.TABLE_A.ID_A,PUBLIC.TABLE_A.NAME_A,PUBLIC.TABLE_B.ID_B,PUBLIC.TABLE_B.NAME_B FROM PUBLIC.TABLE_A,PUBLIC.TABLE_B",
             new Object[][] {
             new Object[] {
-                new Integer(1), "first A", new Integer(1), "first B"
+                Integer.valueOf(1), "first A", Integer.valueOf(1), "first B"
             }, new Object[] {
-                new Integer(1), "first A", new Integer(2), "second B"
+                Integer.valueOf(1), "first A", Integer.valueOf(2), "second B"
             }, new Object[] {
-                new Integer(2), "second A", new Integer(1), "first B"
+                Integer.valueOf(2), "second A", Integer.valueOf(1), "first B"
             }, new Object[] {
-                new Integer(2), "second A", new Integer(2), "second B"
+                Integer.valueOf(2), "second A", Integer.valueOf(2), "second B"
             }
         });
 
@@ -440,13 +440,13 @@ public class TestViewAsterisks extends TestBase {
             "SELECT  PUBLIC.TABLE_B.ID_B,PUBLIC.TABLE_B.NAME_B , PUBLIC.TABLE_A.ID_A,PUBLIC.TABLE_A.NAME_A  FROM PUBLIC.TABLE_A,PUBLIC.TABLE_B",
             new Object[][] {
             new Object[] {
-                new Integer(1), "first B", new Integer(1), "first A"
+                Integer.valueOf(1), "first B", Integer.valueOf(1), "first A"
             }, new Object[] {
-                new Integer(2), "second B", new Integer(1), "first A"
+                Integer.valueOf(2), "second B", Integer.valueOf(1), "first A"
             }, new Object[] {
-                new Integer(1), "first B", new Integer(2), "second A"
+                Integer.valueOf(1), "first B", Integer.valueOf(2), "second A"
             }, new Object[] {
-                new Integer(2), "second B", new Integer(2), "second A"
+                Integer.valueOf(2), "second B", Integer.valueOf(2), "second A"
             }
         });
 
@@ -456,13 +456,13 @@ public class TestViewAsterisks extends TestBase {
             "SELECT PUBLIC.TABLE_A.ID_A,PUBLIC.TABLE_A.NAME_A FROM PUBLIC.TABLE_A,PUBLIC.TABLE_B",
             new Object[][] {
             new Object[] {
-                new Integer(1), "first A"
+                Integer.valueOf(1), "first A"
             }, new Object[] {
-                new Integer(1), "first A"
+                Integer.valueOf(1), "first A"
             }, new Object[] {
-                new Integer(2), "second A"
+                Integer.valueOf(2), "second A"
             }, new Object[] {
-                new Integer(2), "second A"
+                Integer.valueOf(2), "second A"
             }
         });
     }
@@ -538,13 +538,13 @@ public class TestViewAsterisks extends TestBase {
             "SELECT PUBLIC.TABLE_A.ID_A,PUBLIC.TABLE_A.NAME_A FROM PUBLIC.TABLE_A UNION SELECT PUBLIC.TABLE_B.ID_B,PUBLIC.TABLE_B.NAME_B FROM PUBLIC.TABLE_B",
             new Object[][] {
             new Object[] {
-                new Integer(1), "first A"
+                Integer.valueOf(1), "first A"
             }, new Object[] {
-                new Integer(1), "first B"
+                Integer.valueOf(1), "first B"
             }, new Object[] {
-                new Integer(2), "second A"
+                Integer.valueOf(2), "second A"
             }, new Object[] {
-                new Integer(2), "second B"
+                Integer.valueOf(2), "second B"
             }
         });
         checkViewTranslationAndContent(
@@ -553,13 +553,13 @@ public class TestViewAsterisks extends TestBase {
             "SELECT ID_A,NAME_A FROM(SELECT PUBLIC.TABLE_A.ID_A,PUBLIC.TABLE_A.NAME_A FROM PUBLIC.TABLE_A UNION SELECT PUBLIC.TABLE_B.ID_B,PUBLIC.TABLE_B.NAME_B FROM PUBLIC.TABLE_B)",
             new Object[][] {
             new Object[] {
-                new Integer(1), "first A"
+                Integer.valueOf(1), "first A"
             }, new Object[] {
-                new Integer(1), "first B"
+                Integer.valueOf(1), "first B"
             }, new Object[] {
-                new Integer(2), "second A"
+                Integer.valueOf(2), "second A"
             }, new Object[] {
-                new Integer(2), "second B"
+                Integer.valueOf(2), "second B"
             }
         });
     }
