@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -121,7 +121,7 @@ public class BinaryType extends Type {
             return getNameString();
         }
 
-        StringBuffer sb = new StringBuffer(16);
+        StringBuilder sb = new StringBuilder(16);
 
         sb.append(getNameString());
         sb.append('(');
@@ -626,7 +626,7 @@ public class BinaryType extends Type {
             throw Error.error(ErrorCode.X_22011);
         }
 
-        if (offset > end || end < 0) {
+        if (end < 0) {
 
             // return zero length data
             offset = 0;
