@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,7 @@ import org.hsqldb.rights.Grantee;
  * Implementation of CHARACTER SET objects.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.5.0
  * @since 1.9.0
  */
 public class Charset implements SchemaObject {
@@ -151,14 +151,20 @@ public class Charset implements SchemaObject {
          */
     }
 
-    HsqlName        name;
-    public HsqlName base;
+    final HsqlName        name;
+    final public HsqlName base;
 
     //
     int[][] ranges;
 
-    public Charset(HsqlName name) {
+    private Charset(HsqlName name) {
         this.name = name;
+        this.base = null;
+    }
+
+    public Charset(HsqlName name, HsqlName baseName) {
+        this.name = name;
+        this.base = baseName;
     }
 
     public int getType() {
@@ -185,7 +191,9 @@ public class Charset implements SchemaObject {
 
         OrderedHashSet set = new OrderedHashSet();
 
-        set.add(base);
+        if (base != null) {
+            set.add(base);
+        }
 
         return set;
     }
@@ -198,7 +206,7 @@ public class Charset implements SchemaObject {
 
     public String getSQL() {
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
 
         sb.append(Tokens.T_CREATE).append(' ').append(
             Tokens.T_CHARACTER).append(' ').append(Tokens.T_SET).append(' ');
@@ -210,7 +218,8 @@ public class Charset implements SchemaObject {
         }
 
         if (base != null) {
-            sb.append(' ').append(Tokens.T_AS).append(' ').append(Tokens.T_GET);
+            sb.append(' ').append(Tokens.T_AS).append(' ').append(
+                Tokens.T_GET);
             sb.append(' ');
 
             if (SqlInvariants.INFORMATION_SCHEMA.equals(base.schema.name)) {
