@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2018, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,13 +34,15 @@ package org.hsqldb.util;
 import java.applet.Applet;
 import java.io.File;
 import java.io.IOException;
+import java.io.PrintWriter;
 import java.sql.Connection;
 import java.sql.DatabaseMetaData;
+import java.sql.DriverManager;
 import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 import java.sql.Statement;
-import java.util.Vector;
+import java.util.ArrayList;
 
 import java.awt.BorderLayout;
 import java.awt.Button;
@@ -66,13 +68,14 @@ import java.awt.event.WindowListener;
 import java.awt.image.MemoryImageSource;
 
 import org.hsqldb.lib.RCData;
-import org.hsqldb.lib.java.JavaSystem;
 
 // sqlbob@users 20020401 - patch 1.7.0 by sqlbob (RMP) - enhancements
 // sqlbob@users 20020401 - patch 537501 by ulrivo - command line arguments
 // sqlbob@users 20020407 - patch 1.7.0 - reengineering
 // nickferguson@users 20021005 - patch 1.7.1 - enhancements
+// fredt@users - version 2.50 - removed deprecated
 /*
+
  * unsaved@users 20050426 - Switched default switch method from "-switch" to
  * "--switch" because "-switch" usage is ambiguous as used here.  Single
  * switches should be reserved for single-letter switches which can be mixed
@@ -99,7 +102,7 @@ import org.hsqldb.lib.java.JavaSystem;
  * Originally in HypersonicSQL. Extended in various versions of HSQLDB.
  *
  * @author Thomas Mueller (Hypersonic SQL Group)
- * @version 2.4.1
+ * @version 2.5.0
  * @since Hypersonic SQL
  */
 public class DatabaseManager extends Applet
@@ -122,15 +125,15 @@ implements ActionListener, WindowListener, KeyListener {
         "See the forums, mailing lists, and HSQLDB User Guide\n"
         + "at http://hsqldb.org.\n\n"
         + "Please paste the following version identifier with any\n"
-        + "problem reports or help requests:  $Revision: 5790 $"
+        + "problem reports or help requests:  $Revision: 5982 $"
         + (TT_AVAILABLE ? ""
                         : ("\n\nTransferTool classes are not in CLASSPATH.\n"
                            + "To enable the Tools menu, add 'transfer.jar' to your class path."));
     ;
     private static final String ABOUT_TEXT =
-        "$Revision: 5790 $ of DatabaseManager\n\n"
+        "$Revision: 5982 $ of DatabaseManager\n\n"
         + "Copyright (c) 1995-2000, The Hypersonic SQL Group.\n"
-        + "Copyright (c) 2001-2018, The HSQL Development Group.\n"
+        + "Copyright (c) 2001-2019, The HSQL Development Group.\n"
         + "http://hsqldb.org  (User Guide available at this site).\n\n\n"
         + "You may use and redistribute according to the HSQLDB\n"
         + "license documented in the source code and at the web\n"
@@ -312,7 +315,7 @@ implements ActionListener, WindowListener, KeyListener {
                  */
                 throw new IllegalArgumentException(
                     "invalid argrument " + currentArg + " try:  java... "
-                    + DatabaseManagerSwing.class.getName() + " --help");
+                    + DatabaseManager.class.getName() + " --help");
 
                 // No reason to localize, since the main syntax message is
                 // not localized.
@@ -488,7 +491,7 @@ implements ActionListener, WindowListener, KeyListener {
             fMain.setSize(d);
         }
 
-        fMain.show();
+        fMain.setVisible(true);
 
         // (ulrivo): load query from command line
         if (defScript != null) {
@@ -573,9 +576,9 @@ implements ActionListener, WindowListener, KeyListener {
             Transfer.work(new String[]{ "-r" });
             refreshTree();
         } else if (s.equals("Logging on")) {
-            JavaSystem.setLogToSystem(true);
+            setLogToSystem(true);
         } else if (s.equals("Logging off")) {
-            JavaSystem.setLogToSystem(false);
+            setLogToSystem(false);
         } else if (s.equals("Help")) {
             showHelp(new String[] {
                 "", HELP_TEXT
@@ -608,12 +611,12 @@ implements ActionListener, WindowListener, KeyListener {
                 f.setDirectory(defDirectory);
             }
 
-            f.show();
+            f.setVisible(true);
 
             String file = f.getFile();
 
             if (file != null) {
-                StringBuffer buf = new StringBuffer();
+                StringBuilder buf = new StringBuilder();
 
                 ifHuge = DatabaseManagerCommon.readFile(f.getDirectory()
                         + file);
@@ -635,7 +638,7 @@ implements ActionListener, WindowListener, KeyListener {
                 f.setDirectory(defDirectory);
             }
 
-            f.show();
+            f.setVisible(true);
 
             String file = f.getFile();
 
@@ -652,7 +655,7 @@ implements ActionListener, WindowListener, KeyListener {
                 f.setDirectory(defDirectory);
             }
 
-            f.show();
+            f.setVisible(true);
 
             String dir  = f.getDirectory();
             String file = f.getFile();
@@ -674,7 +677,7 @@ implements ActionListener, WindowListener, KeyListener {
                 f.setDirectory(defDirectory);
             }
 
-            f.show();
+            f.setVisible(true);
 
             String file = f.getFile();
 
@@ -944,10 +947,11 @@ implements ActionListener, WindowListener, KeyListener {
             ResultSetMetaData m   = r.getMetaData();
             int               col = m.getColumnCount();
             String[]          h   = new String[col];
-            // int[]             p   = new int[col];
 
+            // int[]             p   = new int[col];
             for (int i = 1; i <= col; i++) {
                 h[i - 1] = m.getColumnLabel(i);
+
                 // p[i - 1] = m.getPrecision(i);
             }
 
@@ -975,9 +979,9 @@ implements ActionListener, WindowListener, KeyListener {
 
     void testPerformance() {
 
-        String       all   = txtCommand.getText();
-        StringBuffer b     = new StringBuffer();
-        long         total = 0;
+        String        all   = txtCommand.getText();
+        StringBuilder b     = new StringBuilder();
+        long          total = 0;
 
         lTime = 0;
 
@@ -1061,14 +1065,14 @@ implements ActionListener, WindowListener, KeyListener {
             CSVWriter writer = new CSVWriter(file, null);
             String[]  col    = gResult.getHead();
             int       width  = col.length;
-            Vector    data   = gResult.getData();
+            ArrayList<String[]> data   = gResult.getData();
             String[]  row;
             int       height = data.size();
 
             writer.writeHeader(col);
 
             for (int i = 0; i < height; i++) {
-                row = (String[]) data.elementAt(i);
+                row = data.get(i);
 
                 String[] myRow = new String[row.length];
 
@@ -1098,7 +1102,7 @@ implements ActionListener, WindowListener, KeyListener {
         String[] col   = gResult.getHead();
         int      width = col.length;
         int[]    size  = new int[width];
-        Vector   data  = gResult.getData();
+        ArrayList<String[]> data = gResult.getData();
         String[] row;
         int      height = data.size();
 
@@ -1107,7 +1111,7 @@ implements ActionListener, WindowListener, KeyListener {
         }
 
         for (int i = 0; i < height; i++) {
-            row = (String[]) data.elementAt(i);
+            row = data.get(i);
 
             for (int j = 0; j < width; j++) {
                 int l = row[j].length();
@@ -1118,7 +1122,7 @@ implements ActionListener, WindowListener, KeyListener {
             }
         }
 
-        StringBuffer b = new StringBuffer();
+        StringBuilder b = new StringBuilder();
 
         for (int i = 0; i < width; i++) {
             b.append(col[i]);
@@ -1141,7 +1145,7 @@ implements ActionListener, WindowListener, KeyListener {
         b.append(NL);
 
         for (int i = 0; i < height; i++) {
-            row = (String[]) data.elementAt(i);
+            row = data.get(i);
 
             for (int j = 0; j < width; j++) {
                 b.append(row[j]);
@@ -1267,32 +1271,32 @@ implements ActionListener, WindowListener, KeyListener {
             };
 
             // fredt@users Schema support
-            Vector schemas = new Vector();
-            Vector tables  = new Vector();
+            ArrayList<String> schemas = new ArrayList<String>();
+            ArrayList<String> tables  = new ArrayList<String>();
 
             // sqlbob@users Added remarks.
-            Vector    remarks = new Vector();
+            ArrayList<String> remarks = new ArrayList<String>();
             ResultSet result  = dMeta.getTables(null, null, null, usertables);
 
             try {
                 while (result.next()) {
-                    schemas.addElement(result.getString(2));
-                    tables.addElement(result.getString(3));
-                    remarks.addElement(result.getString(5));
+                    schemas.add(result.getString(2));
+                    tables.add(result.getString(3));
+                    remarks.add(result.getString(5));
                 }
             } finally {
                 result.close();
             }
 
             for (int i = 0; i < tables.size(); i++) {
-                String name   = (String) tables.elementAt(i);
-                String schema = (String) schemas.elementAt(i);
+                String name   = (String) tables.get(i);
+                String schema = (String) schemas.get(i);
                 String key    = "tab-" + name + "-";
 
                 tTree.addRow(key, name, "+", color_table);
 
                 // sqlbob@users Added remarks.
-                String remark = (String) remarks.elementAt(i);
+                String remark = (String) remarks.get(i);
 
                 if ((schema != null) && !schema.trim().equals("")) {
                     tTree.addRow(key + "s", "schema: " + schema);
@@ -1372,4 +1376,15 @@ implements ActionListener, WindowListener, KeyListener {
 
         tTree.update();
     }
+
+    private static void setLogToSystem(boolean value) {
+
+        try {
+            PrintWriter newPrintWriter = (value) ? new PrintWriter(System.out)
+                                                 : null;
+
+            DriverManager.setLogWriter(newPrintWriter);
+        } catch (Exception e) {}
+    }
+
 }
