@@ -65,7 +65,7 @@ import javax.swing.table.TableModel;
  * have been translated via the internal mapping array. This way,
  * the TableSorter appears to hold another copy of the table
  * with the rows in a different order.
- * <p/>
+ * <p>
  * TableSorter registers itself as a listener to the underlying model,
  * just as the JTable itself would. Events recieved from the model
  * are examined, sometimes manipulated (typically widened), and then
@@ -73,7 +73,7 @@ import javax.swing.table.TableModel;
  * If a change to the model has invalidated the order of TableSorter's
  * rows, a note of this is made and the sorter will resort the
  * rows the next time a value is requested.
- * <p/>
+ * <p>
  * When the tableHeader property is set, either by using the
  * setTableHeader() method or the two argument constructor, the
  * table header may be used as a complete UI for TableSorter.
@@ -96,7 +96,7 @@ import javax.swing.table.TableModel;
  * that are already sorting - giving a way to initiate a compound
  * sort.
  * </ul>
- * <p/>
+ * <p>
  * This is a long overdue rewrite of a class of the same name that
  * first appeared in the swing table demos in 1997.
  *
@@ -534,7 +534,7 @@ public class TableSorter extends AbstractTableModel {
 
             // In a compound sort, make each succesive triangle 20%
             // smaller than the previous one.
-            int dx = (int) (size / 2 * Math.pow(0.8, priority));
+            int dx = (int) ((double) size / 2 * Math.pow(0.8, priority));
             int dy = descending ? dx
                                 : -dx;
 
